#!/usr/bin/env python
#
# Copyright 2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


import random
from gnuradio import gr, gr_unittest, blocks
import pmt


class qa_repack_bits_bb (gr_unittest.TestCase):

    def setUp(self):
        random.seed(0)
        self.tb = gr.top_block()
        self.tsb_key = "length"

    def tearDown(self):
        self.tb = None

    def test_001_simple(self):
        """ Very simple test, 2 bits -> 1 """
        src_data = [0b11, 0b01, 0b10]
        expected_data = [0b1, 0b1, 0b1, 0b0, 0b0, 0b1]
        k = 2
        l = 1
        src = blocks.vector_source_b(src_data, False, 1)
        repack = blocks.repack_bits_bb(k, l)
        sink = blocks.vector_sink_b()
        self.tb.connect(src, repack, sink)
        self.tb.run()
        self.assertEqual(sink.data(), expected_data)

    def test_001_simple_msb(self):
        """ Very simple test, 2 bits -> 1 with MSB set """
        src_data = [0b11, 0b01, 0b10]
        expected_data = [0b1, 0b1, 0b0, 0b1, 0b1, 0b0]
        k = 2
        l = 1
        src = blocks.vector_source_b(src_data, False, 1)
        repack = blocks.repack_bits_bb(k, l, "", False, gr.GR_MSB_FIRST)
        sink = blocks.vector_sink_b()
        self.tb.connect(src, repack, sink)
        self.tb.run()
        self.assertEqual(sink.data(), expected_data)

    def test_002_three(self):
        """ 8 -> 3 """
        src_data = [0b11111101, 0b11111111, 0b11111111]
        expected_data = [0b101, ] + [0b111, ] * 7
        k = 8
        l = 3
        src = blocks.vector_source_b(src_data, False, 1)
        repack = blocks.repack_bits_bb(k, l)
        sink = blocks.vector_sink_b()
        self.tb.connect(src, repack, sink)
        self.tb.run()
        self.assertEqual(sink.data(), expected_data)

    def test_002_three_msb(self):
        """ 8 -> 3 """
        src_data = [0b11111101, 0b11111111, 0b11111111]
        expected_data = [0b111, ] + [0b111, ] + [0b011, ] + [0b111, ] * 5
        k = 8
        l = 3
        src = blocks.vector_source_b(src_data, False, 1)
        repack = blocks.repack_bits_bb(k, l, "", False, gr.GR_MSB_FIRST)
        sink = blocks.vector_sink_b()
        self.tb.connect(src, repack, sink)
        self.tb.run()
        self.assertEqual(sink.data(), expected_data)

    def test_003_lots_of_bytes(self):
        """ Lots and lots of bytes, multiple packer stages """
        src_data = [random.randint(0, 255) for x in range(3 * 5 * 7 * 8 * 10)]
        src = blocks.vector_source_b(src_data, False, 1)
        repack1 = blocks.repack_bits_bb(8, 3)
        repack2 = blocks.repack_bits_bb(3, 5)
        repack3 = blocks.repack_bits_bb(5, 7)
        repack4 = blocks.repack_bits_bb(7, 8)
        sink = blocks.vector_sink_b()
        self.tb.connect(src, repack1, repack2, repack3, repack4, sink)
        self.tb.run()
        self.assertEqual(sink.data(), src_data)

    def test_003_lots_of_bytes_msb(self):
        """ Lots and lots of bytes, multiple packer stages """
        src_data = [random.randint(0, 255) for x in range(3 * 5 * 7 * 8 * 10)]
        src = blocks.vector_source_b(src_data, False, 1)
        repack1 = blocks.repack_bits_bb(8, 3, "", False, gr.GR_MSB_FIRST)
        repack2 = blocks.repack_bits_bb(3, 5, "", False, gr.GR_MSB_FIRST)
        repack3 = blocks.repack_bits_bb(5, 7, "", False, gr.GR_MSB_FIRST)
        repack4 = blocks.repack_bits_bb(7, 8, "", False, gr.GR_MSB_FIRST)
        sink = blocks.vector_sink_b()
        self.tb.connect(src, repack1, repack2, repack3, repack4, sink)
        self.tb.run()
        self.assertEqual(sink.data(), src_data)

    def test_004_three_with_tags(self):
        """ 8 -> 3 """
        src_data = [0b11111101, 0b11111111]
        expected_data = [0b101, ] + [0b111, ] * 4 + [0b001, ]
        k = 8
        l = 3
        src = blocks.vector_source_b(src_data, False, 1)
        repack = blocks.repack_bits_bb(k, l, self.tsb_key)
        sink = blocks.tsb_vector_sink_b(tsb_key=self.tsb_key)
        self.tb.connect(
            src,
            blocks.stream_to_tagged_stream(
                gr.sizeof_char,
                1,
                len(src_data),
                self.tsb_key),
            repack,
            sink)
        self.tb.run()
        self.assertEqual(len(sink.data()), 1)
        self.assertEqual(sink.data()[0], expected_data)

    def test_005_three_with_tags_trailing(self):
        """ 3 -> 8, trailing bits """
        src_data = [0b101, ] + [0b111, ] * 4 + [0b001, ]
        expected_data = [0b11111101, 0b11111111]
        k = 3
        l = 8
        src = blocks.vector_source_b(src_data, False, 1)
        repack = blocks.repack_bits_bb(k, l, self.tsb_key, True)
        sink = blocks.tsb_vector_sink_b(tsb_key=self.tsb_key)
        self.tb.connect(
            src,
            blocks.stream_to_tagged_stream(
                gr.sizeof_char,
                1,
                len(src_data),
                self.tsb_key),
            repack,
            sink)
        self.tb.run()
        self.assertEqual(len(sink.data()), 1)
        self.assertEqual(sink.data()[0], expected_data)


if __name__ == '__main__':
    gr_unittest.run(qa_repack_bits_bb)



#!/usr/bin/env python
#
# Copyright 2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, blocks

import pmt


class test_burst_tagger(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_001(self):
        src_data = (1, 2, 3, 4, 5, 6, 7, 8, 9, 10)
        trg_data = (-1, -1, 1, 1, -1, -1, 1, 1, -1, -1)
        src = blocks.vector_source_i(src_data)
        trg = blocks.vector_source_s(trg_data)
        op = blocks.burst_tagger(gr.sizeof_int)
        snk = blocks.tag_debug(gr.sizeof_int, "burst tagger QA")
        self.tb.connect(src, (op, 0))
        self.tb.connect(trg, (op, 1))
        self.tb.connect(op, snk)
        self.tb.run()

        x = snk.current_tags()
        self.assertEqual(2, x[0].offset)
        self.assertEqual(4, x[1].offset)
        self.assertEqual(6, x[2].offset)
        self.assertEqual(8, x[3].offset)

        self.assertEqual(True, pmt.to_bool(x[0].value))
        self.assertEqual(False, pmt.to_bool(x[1].value))
        self.assertEqual(True, pmt.to_bool(x[2].value))
        self.assertEqual(False, pmt.to_bool(x[3].value))


if __name__ == '__main__':
    gr_unittest.run(test_burst_tagger)



#!/usr/bin/env python
#
# Copyright 2012,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, blocks

from math import sqrt, atan2


class test_type_conversions(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_char_to_float_identity(self):
        src_data = (1, 2, 3, 4, 5)
        expected_data = [1.0, 2.0, 3.0, 4.0, 5.0]
        src = blocks.vector_source_b(src_data)
        op = blocks.char_to_float()
        dst = blocks.vector_sink_f()
        self.tb.connect(src, op, dst)
        self.tb.run()
        self.assertFloatTuplesAlmostEqual(expected_data, dst.data())

    def test_char_to_float_scale(self):
        src_data = (1, 2, 3, 4, 5)
        expected_data = [0.5, 1.0, 1.5, 2.0, 2.5]
        src = blocks.vector_source_b(src_data)
        op = blocks.char_to_float(scale=2.0)
        dst = blocks.vector_sink_f()
        self.tb.connect(src, op, dst)
        self.tb.run()
        self.assertFloatTuplesAlmostEqual(expected_data, dst.data())

    def test_char_to_short(self):
        src_data = (1, 2, 3, 4, 5)
        expected_data = [256, 512, 768, 1024, 1280]
        src = blocks.vector_source_b(src_data)
        op = blocks.char_to_short()
        dst = blocks.vector_sink_s()
        self.tb.connect(src, op, dst)
        self.tb.run()
        self.assertEqual(expected_data, dst.data())

    def test_complex_to_interleaved_char(self):
        src_data = (1 + 2j, 3 + 4j, 5 + 6j, 7 + 8j, 9 + 10j)
        expected_data = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10]
        src = blocks.vector_source_c(src_data)
        op = blocks.complex_to_interleaved_char()
        dst = blocks.vector_sink_b()
        self.tb.connect(src, op, dst)
        self.tb.run()
        self.assertEqual(expected_data, dst.data())

    def test_complex_to_interleaved_short(self):
        src_data = (1 + 2j, 3 + 4j, 5 + 6j, 7 + 8j, 9 + 10j)
        expected_data = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10]
        src = blocks.vector_source_c(src_data)
        op = blocks.complex_to_interleaved_short()
        dst = blocks.vector_sink_s()
        self.tb.connect(src, op, dst)
        self.tb.run()
        self.assertEqual(expected_data, dst.data())

    def test_complex_to_float_1(self):
        src_data = (1 + 2j, 3 + 4j, 5 + 6j, 7 + 8j, 9 + 10j)
        expected_data = [1.0, 3.0, 5.0, 7.0, 9.0]
        src = blocks.vector_source_c(src_data)
        op = blocks.complex_to_float()
        dst = blocks.vector_sink_f()
        self.tb.connect(src, op, dst)
        self.tb.run()
        self.assertFloatTuplesAlmostEqual(expected_data, dst.data())

    def test_complex_to_float_2(self):
        src_data = (1 + 2j, 3 + 4j, 5 + 6j, 7 + 8j, 9 + 10j)
        expected_data1 = [1.0, 3.0, 5.0, 7.0, 9.0]
        expected_data2 = [2.0, 4.0, 6.0, 8.0, 10.0]
        src = blocks.vector_source_c(src_data)
        op = blocks.complex_to_float()
        dst1 = blocks.vector_sink_f()
        dst2 = blocks.vector_sink_f()
        self.tb.connect(src, op)
        self.tb.connect((op, 0), dst1)
        self.tb.connect((op, 1), dst2)
        self.tb.run()
        self.assertFloatTuplesAlmostEqual(expected_data1, dst1.data())
        self.assertFloatTuplesAlmostEqual(expected_data2, dst2.data())

    def test_complex_to_real(self):
        src_data = (1 + 2j, 3 + 4j, 5 + 6j, 7 + 8j, 9 + 10j)
        expected_data = [1.0, 3.0, 5.0, 7.0, 9.0]
        src = blocks.vector_source_c(src_data)
        op = blocks.complex_to_real()
        dst = blocks.vector_sink_f()
        self.tb.connect(src, op, dst)
        self.tb.run()
        self.assertFloatTuplesAlmostEqual(expected_data, dst.data())

    def test_complex_to_imag(self):
        src_data = (1 + 2j, 3 + 4j, 5 + 6j, 7 + 8j, 9 + 10j)
        expected_data = [2.0, 4.0, 6.0, 8.0, 10.0]
        src = blocks.vector_source_c(src_data)
        op = blocks.complex_to_imag()
        dst = blocks.vector_sink_f()
        self.tb.connect(src, op, dst)
        self.tb.run()
        self.assertFloatTuplesAlmostEqual(expected_data, dst.data())

    def test_complex_to_mag(self):
        src_data = (1 + 2j, 3 - 4j, 5 + 6j, 7 - 8j, -9 + 10j)
        expected_data = [sqrt(5), sqrt(25), sqrt(61), sqrt(113), sqrt(181)]
        src = blocks.vector_source_c(src_data)
        op = blocks.complex_to_mag()
        dst = blocks.vector_sink_f()
        self.tb.connect(src, op, dst)
        self.tb.run()
        self.assertFloatTuplesAlmostEqual(expected_data, dst.data(), 5)

    def test_complex_to_mag_squared(self):
        src_data = (1 + 2j, 3 - 4j, 5 + 6j, 7 - 8j, -9 + 10j)
        expected_data = [5.0, 25.0, 61.0, 113.0, 181.0]
        src = blocks.vector_source_c(src_data)
        op = blocks.complex_to_mag_squared()
        dst = blocks.vector_sink_f()
        self.tb.connect(src, op, dst)
        self.tb.run()
        self.assertFloatTuplesAlmostEqual(expected_data, dst.data())

    def test_complex_to_arg(self):
        src_data = (1 + 2j, 3 - 4j, 5 + 6j, 7 - 8j, -9 + 10j)
        expected_data = [atan2(2, 1), atan2(-4, 3),
                         atan2(6, 5), atan2(-8, 7), atan2(10, -9)]
        src = blocks.vector_source_c(src_data)
        op = blocks.complex_to_arg()
        dst = blocks.vector_sink_f()
        self.tb.connect(src, op, dst)
        self.tb.run()
        self.assertFloatTuplesAlmostEqual(expected_data, dst.data(), 2)

    def test_float_to_char_identity(self):
        src_data = (1.0, 2.0, 3.0, 4.0, 5.0)
        expected_data = [1, 2, 3, 4, 5]
        src = blocks.vector_source_f(src_data)
        op = blocks.float_to_char()
        dst = blocks.vector_sink_b()
        self.tb.connect(src, op, dst)
        self.tb.run()
        self.assertEqual(expected_data, dst.data())

    def test_float_to_char_scale(self):
        src_data = (1.0, 2.0, 3.0, 4.0, 5.0)
        expected_data = [5, 10, 15, 20, 25]
        src = blocks.vector_source_f(src_data)
        op = blocks.float_to_char(1, 5)
        dst = blocks.vector_sink_b()
        self.tb.connect(src, op, dst)
        self.tb.run()
        self.assertEqual(expected_data, dst.data())

    def test_float_to_complex_1(self):
        src_data = (1.0, 3.0, 5.0, 7.0, 9.0)
        expected_data = [1 + 0j, 3 + 0j, 5 + 0j, 7 + 0j, 9 + 0j]
        src = blocks.vector_source_f(src_data)
        op = blocks.float_to_complex()
        dst = blocks.vector_sink_c()
        self.tb.connect(src, op, dst)
        self.tb.run()
        self.assertFloatTuplesAlmostEqual(expected_data, dst.data())

    def test_float_to_complex_2(self):
        src1_data = (1.0, 3.0, 5.0, 7.0, 9.0)
        src2_data = (2.0, 4.0, 6.0, 8.0, 10.0)
        expected_data = [1 + 2j, 3 + 4j, 5 + 6j, 7 + 8j, 9 + 10j]
        src1 = blocks.vector_source_f(src1_data)
        src2 = blocks.vector_source_f(src2_data)
        op = blocks.float_to_complex()
        dst = blocks.vector_sink_c()
        self.tb.connect(src1, (op, 0))
        self.tb.connect(src2, (op, 1))
        self.tb.connect(op, dst)
        self.tb.run()
        self.assertFloatTuplesAlmostEqual(expected_data, dst.data())

    def test_float_to_int_identity(self):
        src_data = (1.0, 2.0, 3.0, 4.0, 5.0, float.fromhex('0x1.0p31'))
        expected_data = [1, 2, 3, 4, 5, 2147483647]
        src = blocks.vector_source_f(src_data)
        op = blocks.float_to_int()
        dst = blocks.vector_sink_i()
        self.tb.connect(src, op, dst)
        self.tb.run()
        self.assertEqual(expected_data, dst.data())

    def test_float_to_int_scale(self):
        src_data = (1.0, 2.0, 3.0, 4.0, 5.0)
        expected_data = [5, 10, 15, 20, 25]
        src = blocks.vector_source_f(src_data)
        op = blocks.float_to_int(1, 5)
        dst = blocks.vector_sink_i()
        self.tb.connect(src, op, dst)
        self.tb.run()
        self.assertEqual(expected_data, dst.data())

    def test_float_to_short_identity(self):
        src_data = (1.0, 2.0, 3.0, 4.0, 5.0)
        expected_data = [1, 2, 3, 4, 5]
        src = blocks.vector_source_f(src_data)
        op = blocks.float_to_short()
        dst = blocks.vector_sink_s()
        self.tb.connect(src, op, dst)
        self.tb.run()
        self.assertEqual(expected_data, dst.data())

    def test_float_to_short_scale(self):
        src_data = (1.0, 2.0, 3.0, 4.0, 5.0)
        expected_data = [5, 10, 15, 20, 25]
        src = blocks.vector_source_f(src_data)
        op = blocks.float_to_short(1, 5)
        dst = blocks.vector_sink_s()
        self.tb.connect(src, op, dst)
        self.tb.run()
        self.assertEqual(expected_data, dst.data())

    def test_float_to_uchar(self):
        src_data = (1.0, -2.0, 3.0, -4.0, 256.0)
        expected_data = [1, 0, 3, 0, 255]
        src = blocks.vector_source_f(src_data)
        op = blocks.float_to_uchar()
        dst = blocks.vector_sink_b()
        self.tb.connect(src, op, dst)
        self.tb.run()
        self.assertEqual(expected_data, dst.data())

    def test_float_to_uchar_scale_bias(self):
        src_data = (1.0, -2.0, 3.0, -4.0, 256.0)
        scale = 2.0
        bias = 5.0
        expected_data = [7, 1, 11, 0, 255]
        src = blocks.vector_source_f(src_data)
        op = blocks.float_to_uchar(scale=scale, bias=bias)
        dst = blocks.vector_sink_b()
        self.tb.connect(src, op, dst)
        self.tb.run()
        self.assertEqual(expected_data, dst.data())

    def test_int_to_float_identity(self):
        src_data = (1, 2, 3, 4, 5)
        expected_data = [1.0, 2.0, 3.0, 4.0, 5.0]
        src = blocks.vector_source_i(src_data)
        op = blocks.int_to_float()
        dst = blocks.vector_sink_f()
        self.tb.connect(src, op, dst)
        self.tb.run()
        self.assertFloatTuplesAlmostEqual(expected_data, dst.data())

    def test_int_to_float_scale(self):
        src_data = (1, 2, 3, 4, 5)
        expected_data = [0.2, 0.4, 0.6, 0.8, 1.0]
        src = blocks.vector_source_i(src_data)
        op = blocks.int_to_float(1, 5)
        dst = blocks.vector_sink_f()
        self.tb.connect(src, op, dst)
        self.tb.run()
        self.assertFloatTuplesAlmostEqual(expected_data, dst.data())

    def test_interleaved_short_to_complex(self):
        src_data = (1, 2, 3, 4, 5, 6, 7, 8, 9, 10)
        expected_data = [1 + 2j, 3 + 4j, 5 + 6j, 7 + 8j, 9 + 10j]
        src = blocks.vector_source_s(src_data)
        op = blocks.interleaved_short_to_complex()
        dst = blocks.vector_sink_c()
        self.tb.connect(src, op, dst)
        self.tb.run()
        self.assertEqual(expected_data, dst.data())

    def test_short_to_char(self):
        src_data = (256, 512, 768, 1024, 1280)
        expected_data = [1, 2, 3, 4, 5]
        src = blocks.vector_source_s(src_data)
        op = blocks.short_to_char()
        dst = blocks.vector_sink_b()
        self.tb.connect(src, op, dst)
        self.tb.run()
        self.assertEqual(expected_data, dst.data())

    def test_short_to_float_identity(self):
        src_data = (1, 2, 3, 4, 5)
        expected_data = [1.0, 2.0, 3.0, 4.0, 5.0]
        src = blocks.vector_source_s(src_data)
        op = blocks.short_to_float()
        dst = blocks.vector_sink_f()
        self.tb.connect(src, op, dst)
        self.tb.run()
        self.assertEqual(expected_data, dst.data())

    def test_short_to_float_scale(self):
        src_data = (5, 10, 15, 20, 25)
        expected_data = [1.0, 2.0, 3.0, 4.0, 5.0]
        src = blocks.vector_source_s(src_data)
        op = blocks.short_to_float(1, 5)
        dst = blocks.vector_sink_f()
        self.tb.connect(src, op, dst)
        self.tb.run()
        self.assertEqual(expected_data, dst.data())

    def test_uchar_to_float(self):
        src_data = (1, 2, 3, 4, 5)
        expected_data = [1.0, 2.0, 3.0, 4.0, 5.0]
        src = blocks.vector_source_b(src_data)
        op = blocks.uchar_to_float()
        dst = blocks.vector_sink_f()
        self.tb.connect(src, op, dst)
        self.tb.run()
        self.assertEqual(expected_data, dst.data())


if __name__ == '__main__':
    gr_unittest.run(test_type_conversions)



#!/usr/bin/env python
#
# Copyright 2021 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later

from gnuradio import gr, gr_unittest, blocks
from gnuradio import soapy


class test_soapy_types(gr_unittest.TestCase):

    def test_range(self):
        test_range = soapy.range_t()
        self.assertAlmostEqual(test_range.minimum(), 0.0, 9)
        self.assertAlmostEqual(test_range.maximum(), 0.0, 9)
        # Don't check step, uninitialized in old Soapy versions

        test_range = soapy.range_t(-1.23456, 1.23456)
        self.assertAlmostEqual(test_range.minimum(), -1.23456, 9)
        self.assertAlmostEqual(test_range.maximum(), 1.23456, 9)
        self.assertAlmostEqual(test_range.step(), 0.0, 9)

        test_range = soapy.range_t(-7.89012, 7.89012, 0.2468)
        self.assertAlmostEqual(test_range.minimum(), -7.89012, 9)
        self.assertAlmostEqual(test_range.maximum(), 7.89012, 9)
        self.assertAlmostEqual(test_range.step(), 0.2468, 9)

    def test_arginfo(self):
        test_arginfo = soapy.arginfo_t()

        test_arginfo.key = "testkey"
        self.assertEqual(test_arginfo.key, "testkey")

        test_arginfo.name = "testname"
        self.assertEqual(test_arginfo.name, "testname")

        test_arginfo.description = "testdescription"
        self.assertEqual(test_arginfo.description, "testdescription")

        test_arginfo.units = "testunits"
        self.assertEqual(test_arginfo.units, "testunits")

        for val in [soapy.argtype_t.BOOL, soapy.argtype_t.INT, soapy.argtype_t.FLOAT, soapy.argtype_t.STRING]:
            test_arginfo.type = val
            self.assertEqual(test_arginfo.type, val)

        test_arginfo.range = soapy.range_t(1, 2, 0.5)
        self.assertAlmostEqual(test_arginfo.range.minimum(), 1, 9)
        self.assertAlmostEqual(test_arginfo.range.maximum(), 2, 9)
        self.assertAlmostEqual(test_arginfo.range.step(), 0.5, 9)

        test_arginfo.options = ["opt1", "opt2", "opt3"]
        self.assertEqual(len(test_arginfo.options), 3)
        self.assertEqual(test_arginfo.options[0], "opt1")
        self.assertEqual(test_arginfo.options[1], "opt2")
        self.assertEqual(test_arginfo.options[2], "opt3")

        test_arginfo.option_names = ["Option1", "Option2", "Option3"]
        self.assertEqual(len(test_arginfo.option_names), 3)
        self.assertEqual(test_arginfo.option_names[0], "Option1")
        self.assertEqual(test_arginfo.option_names[1], "Option2")
        self.assertEqual(test_arginfo.option_names[2], "Option3")

        #
        # Test all value types
        #

        test_arginfo.value = "testvalue"
        self.assertEqual(test_arginfo.type, soapy.argtype_t.STRING)
        self.assertEqual(type(test_arginfo.value), str)
        self.assertEqual(test_arginfo.value, "testvalue")

        test_arginfo.value = False
        self.assertEqual(test_arginfo.type, soapy.argtype_t.BOOL)
        self.assertEqual(type(test_arginfo.value), bool)
        self.assertFalse(test_arginfo.value)

        test_arginfo.value = 100
        self.assertEqual(test_arginfo.type, soapy.argtype_t.INT)
        self.assertEqual(type(test_arginfo.value), int)
        self.assertEqual(test_arginfo.value, 100)

        test_arginfo.value = 1.23
        self.assertEqual(test_arginfo.type, soapy.argtype_t.FLOAT)
        self.assertEqual(type(test_arginfo.value), float)
        self.assertAlmostEqual(test_arginfo.value, 1.23, 6)


if __name__ == '__main__':
    gr_unittest.run(test_soapy_types)



#!/usr/bin/env python
#
# Copyright 2004,2005,2007,2010,2012,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, filter, blocks

import sys
import random


def make_random_complex_tuple(L):
    return [complex(2 * random.random() - 1,
                    2 * random.random() - 1) for _ in range(L)]


def make_random_float_tuple(L):
    return [(2 * random.random() - 1) for _ in range(L)]


def reference_filter_ccc(dec, taps, input):
    """
    compute result using conventional fir filter
    """
    tb = gr.top_block()
    #src = blocks.vector_source_c(((0,) * (len(taps) - 1)) + input)
    src = blocks.vector_source_c(input)
    op = filter.fir_filter_ccc(dec, taps)
    dst = blocks.vector_sink_c()
    tb.connect(src, op, dst)
    tb.run()
    return dst.data()


def reference_filter_fff(dec, taps, input):
    """
    compute result using conventional fir filter
    """
    tb = gr.top_block()
    #src = blocks.vector_source_f(((0,) * (len(taps) - 1)) + input)
    src = blocks.vector_source_f(input)
    op = filter.fir_filter_fff(dec, taps)
    dst = blocks.vector_sink_f()
    tb.connect(src, op, dst)
    tb.run()
    return dst.data()


def reference_filter_ccf(dec, taps, input):
    """
    compute result using conventional fir filter
    """
    tb = gr.top_block()
    src = blocks.vector_source_c(input)
    op = filter.fir_filter_ccf(dec, taps)
    dst = blocks.vector_sink_c()
    tb.connect(src, op, dst)
    tb.run()
    return dst.data()


def print_complex(x):
    for i in x:
        i = complex(i)
        sys.stdout.write("(%6.3f,%6.3fj), " % (i.real, i.imag))
    sys.stdout.write('\n')


class test_fft_filter(gr_unittest.TestCase):

    def setUp(self):
        random.seed(0)

    def tearDown(self):
        pass

    def assert_fft_ok2(self, expected_result, result_data):
        expected_result = expected_result[:len(result_data)]
        self.assertComplexTuplesAlmostEqual2(expected_result, result_data,
                                             abs_eps=1e-9, rel_eps=4e-4)

    def assert_fft_float_ok2(
            self,
            expected_result,
            result_data,
            abs_eps=1e-9,
            rel_eps=4e-4):
        expected_result = expected_result[:len(result_data)]
        self.assertFloatTuplesAlmostEqual2(expected_result, result_data,
                                           abs_eps, rel_eps)

    def test_ccc_001(self):
        tb = gr.top_block()
        src_data = (0, 1, 2, 3, 4, 5, 6, 7)
        taps = (1,)
        expected_result = tuple([complex(x) for x in (0, 1, 2, 3, 4, 5, 6, 7)])
        src = blocks.vector_source_c(src_data)
        op = filter.fft_filter_ccc(1, taps)
        dst = blocks.vector_sink_c()
        tb.connect(src, op, dst)
        tb.run()
        result_data = dst.data()
        # print 'expected:', expected_result
        # print 'results: ', result_data
        self.assertComplexTuplesAlmostEqual(expected_result, result_data, 5)

    def test_ccc_002(self):
        # Test nthreads
        tb = gr.top_block()
        src_data = (0, 1, 2, 3, 4, 5, 6, 7)
        taps = (2,)
        nthreads = 2
        expected_result = tuple([2 * complex(x)
                                 for x in (0, 1, 2, 3, 4, 5, 6, 7)])
        src = blocks.vector_source_c(src_data)
        op = filter.fft_filter_ccc(1, taps, nthreads)
        dst = blocks.vector_sink_c()
        tb.connect(src, op, dst)
        tb.run()
        result_data = dst.data()
        # print 'expected:', expected_result
        # print 'results: ', result_data
        self.assertComplexTuplesAlmostEqual(expected_result, result_data, 5)

    def test_ccc_003(self):
        tb = gr.top_block()
        src_data = (0, 1, 2, 3, 4, 5, 6, 7)
        taps = (2,)
        expected_result = tuple([2 * complex(x)
                                 for x in (0, 1, 2, 3, 4, 5, 6, 7)])
        src = blocks.vector_source_c(src_data)
        op = filter.fft_filter_ccc(1, taps)
        dst = blocks.vector_sink_c()
        tb.connect(src, op, dst)
        tb.run()
        result_data = dst.data()
        # print 'expected:', expected_result
        # print 'results: ', result_data
        self.assertComplexTuplesAlmostEqual(expected_result, result_data, 5)

    def test_ccc_004(self):
        random.seed(0)
        for i in range(25):
            # sys.stderr.write("\n>>> Loop = %d\n" % (i,))
            src_len = 4 * 1024
            src_data = make_random_complex_tuple(src_len)
            ntaps = int(random.uniform(2, 1000))
            taps = make_random_complex_tuple(ntaps)
            expected_result = reference_filter_ccc(1, taps, src_data)

            src = blocks.vector_source_c(src_data)
            op = filter.fft_filter_ccc(1, taps)
            dst = blocks.vector_sink_c()
            tb = gr.top_block()
            tb.connect(src, op, dst)
            tb.run()
            result_data = dst.data()
            del tb
            self.assert_fft_ok2(expected_result, result_data)

    def test_ccc_005(self):
        random.seed(0)
        for i in range(25):
            # sys.stderr.write("\n>>> Loop = %d\n" % (i,))
            dec = i + 1
            src_len = 4 * 1024
            src_data = make_random_complex_tuple(src_len)
            ntaps = int(random.uniform(2, 100))
            taps = make_random_complex_tuple(ntaps)
            expected_result = reference_filter_ccc(dec, taps, src_data)

            src = blocks.vector_source_c(src_data)
            op = filter.fft_filter_ccc(dec, taps)
            dst = blocks.vector_sink_c()
            tb = gr.top_block()
            tb.connect(src, op, dst)
            tb.run()
            del tb
            result_data = dst.data()

            self.assert_fft_ok2(expected_result, result_data)

    def test_ccc_006(self):
        # Test decimating with nthreads=2
        random.seed(0)
        nthreads = 2
        for i in range(25):
            # sys.stderr.write("\n>>> Loop = %d\n" % (i,))
            dec = i + 1
            src_len = 4 * 1024
            src_data = make_random_complex_tuple(src_len)
            ntaps = int(random.uniform(2, 100))
            taps = make_random_complex_tuple(ntaps)
            expected_result = reference_filter_ccc(dec, taps, src_data)

            src = blocks.vector_source_c(src_data)
            op = filter.fft_filter_ccc(dec, taps, nthreads)
            dst = blocks.vector_sink_c()
            tb = gr.top_block()
            tb.connect(src, op, dst)
            tb.run()
            del tb
            result_data = dst.data()

            self.assert_fft_ok2(expected_result, result_data)

    # ----------------------------------------------------------------
    # test _ccf version
    # ----------------------------------------------------------------

    def test_ccf_001(self):
        tb = gr.top_block()
        src_data = (0, 1, 2, 3, 4, 5, 6, 7)
        taps = (1,)
        expected_result = tuple([complex(x) for x in (0, 1, 2, 3, 4, 5, 6, 7)])
        src = blocks.vector_source_c(src_data)
        op = filter.fft_filter_ccf(1, taps)
        dst = blocks.vector_sink_c()
        tb.connect(src, op, dst)
        tb.run()
        result_data = dst.data()
        # print 'expected:', expected_result
        # print 'results: ', result_data
        self.assertComplexTuplesAlmostEqual(expected_result, result_data, 5)

    def test_ccf_002(self):
        # Test nthreads
        tb = gr.top_block()
        src_data = (0, 1, 2, 3, 4, 5, 6, 7)
        taps = (2,)
        nthreads = 2
        expected_result = tuple([2 * complex(x)
                                 for x in (0, 1, 2, 3, 4, 5, 6, 7)])
        src = blocks.vector_source_c(src_data)
        op = filter.fft_filter_ccf(1, taps, nthreads)
        dst = blocks.vector_sink_c()
        tb.connect(src, op, dst)
        tb.run()
        result_data = dst.data()
        # print 'expected:', expected_result
        # print 'results: ', result_data
        self.assertComplexTuplesAlmostEqual(expected_result, result_data, 5)

    def test_ccf_003(self):
        tb = gr.top_block()
        src_data = (0, 1, 2, 3, 4, 5, 6, 7)
        taps = (2,)
        expected_result = tuple([2 * complex(x)
                                 for x in (0, 1, 2, 3, 4, 5, 6, 7)])
        src = blocks.vector_source_c(src_data)
        op = filter.fft_filter_ccf(1, taps)
        dst = blocks.vector_sink_c()
        tb.connect(src, op, dst)
        tb.run()
        result_data = dst.data()
        # print 'expected:', expected_result
        # print 'results: ', result_data
        self.assertComplexTuplesAlmostEqual(expected_result, result_data, 5)

    def test_ccf_004(self):
        random.seed(0)
        for i in range(25):
            # sys.stderr.write("\n>>> Loop = %d\n" % (i,))
            src_len = 4 * 1024
            src_data = make_random_complex_tuple(src_len)
            ntaps = int(random.uniform(2, 1000))
            taps = make_random_float_tuple(ntaps)
            expected_result = reference_filter_ccf(1, taps, src_data)

            src = blocks.vector_source_c(src_data)
            op = filter.fft_filter_ccf(1, taps)
            dst = blocks.vector_sink_c()
            tb = gr.top_block()
            tb.connect(src, op, dst)
            tb.run()
            result_data = dst.data()
            del tb
            self.assert_fft_ok2(expected_result, result_data)

    def test_ccf_005(self):
        random.seed(0)
        for i in range(25):
            # sys.stderr.write("\n>>> Loop = %d\n" % (i,))
            dec = i + 1
            src_len = 4 * 1024
            src_data = make_random_complex_tuple(src_len)
            ntaps = int(random.uniform(2, 100))
            taps = make_random_float_tuple(ntaps)
            expected_result = reference_filter_ccf(dec, taps, src_data)

            src = blocks.vector_source_c(src_data)
            op = filter.fft_filter_ccf(dec, taps)
            dst = blocks.vector_sink_c()
            tb = gr.top_block()
            tb.connect(src, op, dst)
            tb.run()
            del tb
            result_data = dst.data()

            self.assert_fft_ok2(expected_result, result_data)

    def test_ccf_006(self):
        # Test decimating with nthreads=2
        random.seed(0)
        nthreads = 2
        for i in range(25):
            # sys.stderr.write("\n>>> Loop = %d\n" % (i,))
            dec = i + 1
            src_len = 4 * 1024
            src_data = make_random_complex_tuple(src_len)
            ntaps = int(random.uniform(2, 100))
            taps = make_random_float_tuple(ntaps)
            expected_result = reference_filter_ccf(dec, taps, src_data)

            src = blocks.vector_source_c(src_data)
            op = filter.fft_filter_ccc(dec, taps, nthreads)
            dst = blocks.vector_sink_c()
            tb = gr.top_block()
            tb.connect(src, op, dst)
            tb.run()
            del tb
            result_data = dst.data()

            self.assert_fft_ok2(expected_result, result_data)

    # ----------------------------------------------------------------
    # test _fff version
    # ----------------------------------------------------------------

    def test_fff_001(self):
        tb = gr.top_block()
        src_data = (0, 1, 2, 3, 4, 5, 6, 7)
        taps = (1,)
        expected_result = tuple([float(x) for x in (0, 1, 2, 3, 4, 5, 6, 7)])
        src = blocks.vector_source_f(src_data)
        op = filter.fft_filter_fff(1, taps)
        dst = blocks.vector_sink_f()
        tb.connect(src, op, dst)
        tb.run()
        result_data = dst.data()
        # print 'expected:', expected_result
        # print 'results: ', result_data
        self.assertFloatTuplesAlmostEqual(expected_result, result_data, 5)

    def test_fff_002(self):
        tb = gr.top_block()
        src_data = (0, 1, 2, 3, 4, 5, 6, 7)
        taps = (2,)
        expected_result = tuple([2 * float(x)
                                 for x in (0, 1, 2, 3, 4, 5, 6, 7)])
        src = blocks.vector_source_f(src_data)
        op = filter.fft_filter_fff(1, taps)
        dst = blocks.vector_sink_f()
        tb.connect(src, op, dst)
        tb.run()
        result_data = dst.data()
        # print 'expected:', expected_result
        # print 'results: ', result_data
        self.assertFloatTuplesAlmostEqual(expected_result, result_data, 5)

    def test_fff_003(self):
        # Test 02 with nthreads
        tb = gr.top_block()
        src_data = (0, 1, 2, 3, 4, 5, 6, 7)
        taps = (2,)
        nthreads = 2
        expected_result = tuple([2 * float(x)
                                 for x in (0, 1, 2, 3, 4, 5, 6, 7)])
        src = blocks.vector_source_f(src_data)
        op = filter.fft_filter_fff(1, taps, nthreads)
        dst = blocks.vector_sink_f()
        tb.connect(src, op, dst)
        tb.run()
        result_data = dst.data()
        self.assertFloatTuplesAlmostEqual(expected_result, result_data, 5)

    def xtest_fff_004(self):
        random.seed(0)
        for i in range(25):
            sys.stderr.write("\n>>> Loop = %d\n" % (i,))
            src_len = 4096
            src_data = make_random_float_tuple(src_len)
            ntaps = int(random.uniform(2, 1000))
            taps = make_random_float_tuple(ntaps)
            expected_result = reference_filter_fff(1, taps, src_data)

            src = blocks.vector_source_f(src_data)
            op = filter.fft_filter_fff(1, taps)
            dst = blocks.vector_sink_f()
            tb = gr.top_block()
            tb.connect(src, op, dst)
            tb.run()
            result_data = dst.data()

            # print "src_len =", src_len, " ntaps =", ntaps
            try:
                self.assert_fft_float_ok2(
                    expected_result, result_data, abs_eps=1.0)
            except AssertionError:
                expected = open('expected', 'w')
                for x in expected_result:
                    expected.write(repr(x) + '\n')
                actual = open('actual', 'w')
                for x in result_data:
                    actual.write(repr(x) + '\n')
                raise

    def xtest_fff_005(self):
        random.seed(0)
        for i in range(25):
            sys.stderr.write("\n>>> Loop = %d\n" % (i,))
            src_len = 4 * 1024
            src_data = make_random_float_tuple(src_len)
            ntaps = int(random.uniform(2, 1000))
            taps = make_random_float_tuple(ntaps)
            expected_result = reference_filter_fff(1, taps, src_data)

            src = blocks.vector_source_f(src_data)
            op = filter.fft_filter_fff(1, taps)
            dst = blocks.vector_sink_f()
            tb = gr.top_block()
            tb.connect(src, op, dst)
            tb.run()
            result_data = dst.data()

            self.assert_fft_float_ok2(
                expected_result, result_data, abs_eps=2.0)

    def xtest_fff_006(self):
        random.seed(0)
        for i in range(25):
            sys.stderr.write("\n>>> Loop = %d\n" % (i,))
            dec = i + 1
            src_len = 4 * 1024
            src_data = make_random_float_tuple(src_len)
            ntaps = int(random.uniform(2, 100))
            taps = make_random_float_tuple(ntaps)
            expected_result = reference_filter_fff(dec, taps, src_data)

            src = blocks.vector_source_f(src_data)
            op = filter.fft_filter_fff(dec, taps)
            dst = blocks.vector_sink_f()
            tb = gr.top_block()
            tb.connect(src, op, dst)
            tb.run()
            result_data = dst.data()

            self.assert_fft_float_ok2(expected_result, result_data)

    def xtest_fff_007(self):
        # test decimation with nthreads
        random.seed(0)
        nthreads = 2
        for i in range(25):
            sys.stderr.write("\n>>> Loop = %d\n" % (i,))
            dec = i + 1
            src_len = 4 * 1024
            src_data = make_random_float_tuple(src_len)
            ntaps = int(random.uniform(2, 100))
            taps = make_random_float_tuple(ntaps)
            expected_result = reference_filter_fff(dec, taps, src_data)

            src = blocks.vector_source_f(src_data)
            op = filter.fft_filter_fff(dec, taps, nthreads)
            dst = blocks.vector_sink_f()
            tb = gr.top_block()
            tb.connect(src, op, dst)
            tb.run()
            result_data = dst.data()

            self.assert_fft_float_ok2(expected_result, result_data)

    def test_fff_get0(self):
        random.seed(0)
        for i in range(25):
            ntaps = int(random.uniform(2, 100))
            taps = list(make_random_float_tuple(ntaps))

            op = filter.fft_filter_fff(1, taps)
            result_data = op.taps()
            # print result_data

            self.assertFloatTuplesAlmostEqual(taps, result_data, 4)

    def test_ccc_get0(self):
        random.seed(0)
        for i in range(25):
            ntaps = int(random.uniform(2, 100))
            taps = make_random_complex_tuple(ntaps)

            op = filter.fft_filter_ccc(1, taps)
            result_data = op.taps()
            # print result_data

            self.assertComplexTuplesAlmostEqual(taps, result_data, 4)


if __name__ == '__main__':
    gr_unittest.run(test_fft_filter)



#!/usr/bin/env python
#
# Copyright 2020 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#
"""Unit tests for ZMQ REQ Message Source block"""

import sys
import time
import zmq

from gnuradio import gr, gr_unittest, blocks, zeromq
import pmt


class qa_zeromq_req_msg_source(gr_unittest.TestCase):
    """Unit tests for ZMQ REQ Message Source block"""

    def setUp(self):
        addr = 'tcp://127.0.0.1'
        self.context = zmq.Context()
        self.zmq_sock = self.context.socket(zmq.REP)
        port = self.zmq_sock.bind_to_random_port(addr)

        self.zeromq_req_msg_source = zeromq.req_msg_source(
            ('%s:%s' % (addr, port)), 100)
        self.message_debug = blocks.message_debug()
        self.tb = gr.top_block()
        self.tb.msg_connect(
            (self.zeromq_req_msg_source, 'out'), (self.message_debug, 'store'))

        self.tb.start()

    def tearDown(self):
        self.tb.stop()
        self.tb.wait()
        self.zeromq_req_msg_source = None
        self.message_debug = None
        self.tb = None
        self.zmq_sock.close()
        self.context.term()

    def recv_request(self):
        """Receive the request output items message"""
        req = self.zmq_sock.recv()
        req_output_items = int.from_bytes(req, sys.byteorder)
        self.assertEqual(1, req_output_items)

    def send_messages(self, msgs):
        """Send multiple messages"""
        for msg in msgs:
            self.recv_request()
            self.zmq_sock.send(pmt.serialize_str(msg))

    def test_valid_pmt(self):
        """Test receiving of valid PMT messages"""
        msg = pmt.to_pmt('test_valid_pmt')
        self.send_messages([msg])
        for _ in range(10):
            if self.message_debug.num_messages() > 0:
                break
            time.sleep(0.2)
        self.assertEqual(1, self.message_debug.num_messages())
        self.assertTrue(pmt.equal(msg, self.message_debug.get_message(0)))

    def test_multiple_messages(self):
        """Test receiving of multiple PMT messages"""
        msgs = [pmt.to_pmt('test_valid_pmt msg0'),
                pmt.to_pmt('test_valid_pmt msg1'),
                pmt.to_pmt('test_valid_pmt msg2')]
        self.send_messages(msgs)

        time.sleep(0.1)
        self.assertEqual(len(msgs), self.message_debug.num_messages())
        for index, msg in enumerate(msgs):
            self.assertTrue(
                pmt.equal(
                    msg,
                    self.message_debug.get_message(index)))

    def test_invalid_pmt(self):
        """Test receiving of invalid PMT messages"""
        self.recv_request()

        # Do not use pmt.serialize_str here as we don't want a valid PMT
        # message
        self.zmq_sock.send_string('test_invalid_pmt')

        time.sleep(0.1)
        self.assertEqual(0, self.message_debug.num_messages())


if __name__ == '__main__':
    gr_unittest.run(qa_zeromq_req_msg_source)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 202013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest
from gnuradio import blocks


class qa_stream_to_tagged_stream (gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_001_t(self):
        src_data = [1, ] * 50
        packet_len = 10
        len_tag_key = 'packet_len'
        src = blocks.vector_source_f(src_data, False, 1)
        tagger = blocks.stream_to_tagged_stream(
            gr.sizeof_float, 1, packet_len, len_tag_key)
        sink = blocks.vector_sink_f()
        self.tb.connect(src, tagger, sink)
        self.tb.run()
        self.assertEqual(sink.data(), src_data)
        tags = [gr.tag_to_python(x) for x in sink.tags()]
        tags = sorted([(x.offset, x.key, x.value) for x in tags])
        expected_tags = [(int(pos), 'packet_len', packet_len)
                         for pos in range(0, 50, 10)]
        self.assertEqual(tags, expected_tags)


if __name__ == '__main__':
    gr_unittest.run(qa_stream_to_tagged_stream)



#!/usr/bin/env python3
# Copyright 2022 Daniel Estevez <daniel@destevez.net>
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#


from gnuradio import gr, gr_unittest, blocks, digital
import pmt


class qa_crc(gr_unittest.TestCase):
    def setUp(self):
        """Common part of all CRC tests

        Creates a flowgraph, a Message Debug block, and a PDU
        containing the numbers 0x00 through 0x0F.
        """
        self.tb = gr.top_block()
        self.dbg = blocks.message_debug()
        self.data = list(range(16))
        self.pdu = pmt.cons(pmt.PMT_NIL,
                            pmt.init_u8vector(len(self.data), self.data))

    def run_crc_append(self, crc_params, crc_result):
        """Common part of CRC Append tests

        Creates a CRC Append block with the specified crc_params parameters,
        connects it to the Message Debug block, sends a test PDU to the
        CRC Append block, and checks that the output PDU matches the expected
        crc_result.
        """
        crc_append_block = digital.crc_append(*crc_params)
        self.tb.msg_connect((crc_append_block, 'out'), (self.dbg, 'store'))
        crc_append_block.to_basic_block()._post(pmt.intern('in'), self.pdu)
        crc_append_block.to_basic_block()._post(
            pmt.intern('system'),
            pmt.cons(pmt.intern('done'), pmt.from_long(1)))

        self.tb.start()
        self.tb.wait()

        self.assertEqual(self.dbg.num_messages(), 1)
        out = pmt.u8vector_elements(pmt.cdr(self.dbg.get_message(0)))
        self.assertEqual(out[:len(self.data)], self.data)
        self.assertEqual(out[len(self.data):], crc_result)

    def common_test_crc_check(self, matching_crc, header_bytes=0):
        """Common part of CRC Check tests

        Creates a CRC Append block and a CRC Check block using either the
        same CRC or a different one depending on the whether matching_crc
        is True or False. Connects CRC Append -> CRC Check -> Message Debug
        and sends a PDU through. There are two message debugs to allow
        checking whether the PDU ended up in the ok or fail port of the
        CRC Check block.
        """
        crc_append_block = digital.crc_append(
            16, 0x1021, 0x0, 0x0, False, False, False, header_bytes)
        x = 0x0 if matching_crc else 0xFFFF
        crc_check_block = digital.crc_check(
            16, 0x1021, x, x, False, False, False, True, header_bytes)

        self.dbg_fail = blocks.message_debug()
        self.tb.msg_connect((crc_append_block, 'out'), (crc_check_block, 'in'))
        self.tb.msg_connect((crc_check_block, 'ok'), (self.dbg, 'store'))
        self.tb.msg_connect((crc_check_block, 'fail'),
                            (self.dbg_fail, 'store'))

        crc_append_block.to_basic_block()._post(pmt.intern('in'), self.pdu)
        crc_append_block.to_basic_block()._post(
            pmt.intern('system'),
            pmt.cons(pmt.intern('done'), pmt.from_long(1)))
        self.tb.start()
        self.tb.wait()

    def test_crc_check(self):
        """Test a successful CRC check

        Checks that the PDU ends in the ok port of CRC check
        """
        self.common_test_crc_check(matching_crc=True)
        self.assertEqual(self.dbg.num_messages(), 1)
        out = pmt.u8vector_elements(pmt.cdr(self.dbg.get_message(0)))
        self.assertEqual(out, self.data)
        self.assertEqual(self.dbg_fail.num_messages(), 0)

    def test_crc_check_header_bytes(self):
        """Test a successful CRC check (skipping some header bytes)

        Checks that the PDU ends in the ok port of CRC check
        """
        self.common_test_crc_check(matching_crc=True, header_bytes=5)
        self.assertEqual(self.dbg.num_messages(), 1)
        out = pmt.u8vector_elements(pmt.cdr(self.dbg.get_message(0)))
        self.assertEqual(out, self.data)
        self.assertEqual(self.dbg_fail.num_messages(), 0)

    def test_crc_check_wrong_crc(self):
        """Test a failed CRC check

        Checks that the PDU ends in the fail port of CRC check
        """
        self.common_test_crc_check(matching_crc=False)
        self.assertEqual(self.dbg.num_messages(), 0)
        self.assertEqual(self.dbg_fail.num_messages(), 1)
        out = pmt.u8vector_elements(pmt.cdr(self.dbg_fail.get_message(0)))
        self.assertEqual(out, self.data)

    def test_crc_append_crc16_ccitt_zero(self):
        """Test CRC-16-CCITT-Zero calculation"""
        self.run_crc_append(
            (16, 0x1021, 0x0, 0x0,
             False, False, False),
            [0x51, 0x3D])

    def test_crc_append_crc16_ccitt_false(self):
        """Test CRC-16-CCITT-False calculation"""
        self.run_crc_append(
            (16, 0x1021, 0xFFFF, 0x0,
             False, False, False),
            [0x3B, 0x37])

    def test_crc_append_crc16_ccitt_x25(self):
        """Test CRC-16-CCITT-X.25 calculation"""
        self.run_crc_append(
            (16, 0x1021, 0xFFFF, 0xFFFF,
             True, True, False),
            [0x13, 0xE9])

    def test_crc_append_crc32(self):
        """Test CRC-32 calculation"""
        self.run_crc_append(
            (32, 0x4C11DB7, 0xFFFFFFFF, 0xFFFFFFFF,
             True, True, False),
            [0xCE, 0xCE, 0xE2, 0x88])

    def test_crc_append_crc32c(self):
        """Test CRC-32C calculation"""
        self.run_crc_append(
            (32, 0x1EDC6F41, 0xFFFFFFFF, 0xFFFFFFFF,
             True, True, False),
            [0xD9, 0xC9, 0x08, 0xEB])

    def test_crc_append_crc32c_endianness_swap(self):
        """Test CRC-32C calculation with endianness swapped"""
        self.run_crc_append(
            (32, 0x1EDC6F41, 0xFFFFFFFF, 0xFFFFFFFF,
             True, True, True),
            [0xEB, 0x08, 0xC9, 0xD9])

    def test_crc_append_crc32c_skip_header_bytes(self):
        """Test CRC-32C calculation skipping some header bytes"""
        skip_bytes = 3
        self.run_crc_append(
            (32, 0x1EDC6F41, 0xFFFFFFFF, 0xFFFFFFFF,
             True, True, False, skip_bytes),
            [0xE8, 0x62, 0x60, 0x68])


class qa_crc_class(gr_unittest.TestCase):
    def test_crc_crc32c(self):
        """Test CRC-32C calculation (using crc class directly)"""
        c = digital.crc(32, 0x1EDC6F41, 0xFFFFFFFF, 0xFFFFFFFF, True, True)
        out = c.compute(list(range(16)))
        self.assertEqual(c.compute(list(range(16))),
                         0xD9C908EB)


if __name__ == '__main__':
    gr_unittest.run(qa_crc)
    gr_unittest.run(qa_crc_class)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2021 Caliola Engineering, LLC.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gnuradio import gr, gr_unittest
from gnuradio import blocks


class qa_matrix_interleaver(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_interleave(self):
        tb = self.tb

        # set up fg
        cols, rows = 4, 10
        vec = sum((cols * [x, ] for x in range(rows)), [])
        expected = cols * list(range(rows))

        src = blocks.vector_source_f(vec, False)
        itlv = blocks.matrix_interleaver(gr.sizeof_float, rows=rows, cols=cols)
        snk = blocks.vector_sink_f()

        tb.connect(src, itlv, snk)
        tb.run()
        result = snk.data()

        # check data
        self.assertFloatTuplesAlmostEqual(expected, result)

    def test_deinterleave(self):
        tb = self.tb

        # set up fg
        cols, rows = 4, 10
        vec = sum((rows * [x, ] for x in range(cols)), [])
        expected = rows * list(range(cols))

        src = blocks.vector_source_f(vec, False)
        itlv = blocks.matrix_interleaver(
            gr.sizeof_float, rows=rows, cols=cols, deint=True)
        snk = blocks.vector_sink_f()

        tb.connect(src, itlv, snk)
        tb.run()
        result = snk.data()

        # check data
        self.assertFloatTuplesAlmostEqual(expected, result)


if __name__ == '__main__':
    gr_unittest.run(qa_matrix_interleaver)



# Copyright 2016 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import pytest

from argparse import Namespace
from os import path
import tempfile

from grc.compiler import main


def test_compiler(capsys):
    args = Namespace(
        output=tempfile.gettempdir(),
        user_lib_dir=False,
        grc_files=[path.join(path.dirname(__file__),
                             'resources', 'test_compiler.grc')],
        run=True
    )

    main(args)

    out, err = capsys.readouterr()
    assert not err



#!/usr/bin/env python
# Copyright 2022 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, blocks, digital
import pmt


class qa_crc32_async_bb(gr_unittest.TestCase):
    def setUp(self):
        self.tb = gr.top_block()
        self.tsb_key = "length"

    def tearDown(self):
        self.tb = None

    def test_crc32(self):
        crc_append_block = digital.crc32_async_bb(check=False)
        crc_check_block = digital.crc32_async_bb(check=True)

        dbg_append = blocks.message_debug()
        dbg_check = blocks.message_debug()

        self.tb.msg_connect((crc_append_block, 'out'), (crc_check_block, 'in'))
        self.tb.msg_connect((crc_append_block, 'out'), (dbg_append, 'store'))
        self.tb.msg_connect((crc_check_block, 'out'), (dbg_check, 'store'))

        data = list(range(16))
        pdu = pmt.cons(pmt.PMT_NIL, pmt.init_u8vector(len(data), data))

        crc_append_block._post(pmt.intern('in'), pdu)
        crc_append_block._post(
            pmt.intern('system'),
            pmt.cons(pmt.intern('done'), pmt.from_long(1)))

        self.tb.run()

        self.assertEqual(dbg_append.num_messages(), 1)
        out_append = pmt.u8vector_elements(pmt.cdr(dbg_append.get_message(0)))
        self.assertEqual(out_append, data + [0x88, 0xe2, 0xce, 0xce])

        self.assertEqual(dbg_check.num_messages(), 1)
        out_check = pmt.u8vector_elements(pmt.cdr(dbg_check.get_message(0)))
        self.assertEqual(out_check, data)


if __name__ == '__main__':
    gr_unittest.run(qa_crc32_async_bb)



#!/usr/bin/env python
#
# Copyright 2012,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, analog, blocks, channels
import math


class test_fading_model(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_000(self):
        N = 1000         # number of samples to use
        fs = 1000        # baseband sampling rate
        freq = 100

        fDTs = 0.01
        K = 4
        signal = analog.sig_source_c(fs, analog.GR_SIN_WAVE, freq, 1)
        head = blocks.head(gr.sizeof_gr_complex, N)
        op = channels.fading_model(8, fDTs=fDTs, LOS=True,
                                   K=K, seed=0)
        snk = blocks.vector_sink_c()
        snk1 = blocks.vector_sink_c()

        self.assertAlmostEqual(K, op.K(), 4)
        self.assertAlmostEqual(fDTs, op.fDTs(), 4)

        #self.tb.connect(signal, head, op, snk)
        #self.tb.connect(op, snk1)
        # self.tb.run()

        #dst_data = snk.data()
        #exp_data = snk1.data()
        #self.assertComplexTuplesAlmostEqual(exp_data, dst_data, 5)


if __name__ == '__main__':
    gr_unittest.run(test_fading_model)



#!/usr/bin/env python3

import os
import re
from argparse import ArgumentParser

template_keywords = [
    "GR_EXPAND_X_H", "GR_EXPAND_CC_H", "GR_EXPAND_X_CC_H_IMPL", "GR_EXPAND_X_CC_H"
]
template_regex = re.compile(
    r"^(?P<template_type>" + "|".join(template_keywords) + r")\(" +
    r"(?P<category>\w+)\s+(?P<name>\w+_XX?X?)(_impl)?\s+(?P<types>[\w\s]+)" + r"\)$")

cpp_keywords = ["abs", "add", "and", "max", "min" "not" "xor"]
types = {"s": "std::int16_t", "i": "std::int32_t",
         "b": "std::uint8_t", "c": "gr_complex", "f": "float"}


def get_real_name(block_definition):
    """
    Return the base name of the template blocks (e.g. foo_XXX)
    """
    return "_".join(block_definition.get("name").split("_")[:-1])


def rewrite_cmakelists(block_definition):
    """
    Remove gengen template invocations from CMakeLists.txt
    """
    with open(
            os.path.join(block_definition.get("path"), "CMakeLists.txt"),
            "r") as f:
        cmakelists = f.readlines()
    cmakelists_new = []
    for line in cmakelists:
        if line.startswith(
                block_definition.get("template_type") + "(" + block_definition.
                get("category") + " " + block_definition.get("name")):
            continue
        cmakelists_new.append(line)
    with open(
            os.path.join(block_definition.get("path"), "CMakeLists.txt"),
            "w") as f:
        f.writelines(cmakelists_new)


def convert_public_header(block_definition):
    """
    Replace template arguments with the correct C++ template
    expressions
    """
    real_name = get_real_name(block_definition)
    original_name = real_name
    cpp_keyword = False
    if real_name in cpp_keywords:
        real_name += "_blk"
        cpp_keyword = True
    target_file = os.path.join(block_definition.get("path"), real_name + ".h")
    source_file = os.path.join(
        block_definition.get("path"),
        block_definition.get("name") + ".h.t")
    os.rename(source_file, target_file)

    with open(target_file, "r") as f:
        content = f.readlines()

    with open(target_file, "w") as f:
        new_content = []
        typedefs = False
        for line in content:
            line = line.replace("@GUARD_NAME@", "_".join([real_name,
                                                          "h"]).upper())
            if "typedef" in line:
                line = line.replace("@NAME@", " " + real_name + "<T> ")
                line = line.replace("@BASE_NAME@", " " + real_name + "<T> ")
            else:
                line = line.replace("@NAME@", real_name)
                line = line.replace("@BASE_NAME@", real_name)

            line = line.replace("@I_TYPE@", "T")
            line = line.replace("@O_TYPE@", "T")
            line = line.replace("@TYPE@", "T")
            if "@WARNING@" in line:
                continue
            if "class" in line:
                new_content.append("template<class T>\n")
            if not typedefs and "} /* namespace" in line:
                for t in block_definition.get("types"):
                    new_content.append("typedef " + real_name + "<" +
                                       types[t[0]] + "> " + original_name + "_" + t + ";\n")
                    typedefs = True
            new_content.append(line)
        f.writelines(new_content)


def convert_impl_header(block_definition):
    """
    Replace template arguments with the correct C++ template
    expressions
    """
    real_name = get_real_name(block_definition)
    cpp_keyword = False
    if real_name in cpp_keywords:
        real_name += "_blk"
        cpp_keyword = True
    target_header = os.path.join(
        block_definition.get("path"), real_name + "_impl.h")
    source_header = os.path.join(
        block_definition.get("path"),
        block_definition.get("name") + "_impl.h.t")
    os.rename(source_header, target_header)
    with open(target_header, "r") as f:
        content = f.readlines()

    with open(target_header, "w") as f:
        new_content = []
        for line in content:
            line = line.replace("@GUARD_NAME_IMPL@",
                                "_".join([real_name, "impl_h"]).upper())
            line = line.replace("@GUARD_NAME@",
                                "_".join([real_name, "impl_h"]).upper())
            if "typedef" in line or "class" in line:
                line = line.replace("@NAME@", " " + real_name + "<T> ")
                line = line.replace("@BASE_NAME@", " " + real_name + "<T> ")
                line = line.replace("@NAME_IMPL@", real_name + "_impl<T> ")
                line = line.replace("@IMPL_NAME@", real_name + "_impl<T> ")
            else:
                line = line.replace("@NAME@", real_name)
                line = line.replace("@BASE_NAME@", real_name)
                line = line.replace("@NAME_IMPL@", real_name + "_impl ")
                line = line.replace("@IMPL_NAME@", real_name + "_impl")

            line = line.replace("@I_TYPE@", "T")
            line = line.replace("@O_TYPE@", "T")
            line = line.replace("@TYPE@", "T")
            if "@WARNING@" in line:
                continue
            if "class" in line:
                new_content.append("template<class T>\n")
            new_content.append(line)
        f.writelines(new_content)


def convert_impl_impl(block_definition):
    """
    Replace template arguments with the correct C++ template
    expressions
    """
    real_name = get_real_name(block_definition)
    cpp_keyword = False
    if real_name in cpp_keywords:
        real_name += "_blk"
        cpp_keyword = True

    target_impl = os.path.join(
        block_definition.get("path"), real_name + "_impl.cc")
    source_impl = os.path.join(
        block_definition.get("path"),
        block_definition.get("name") + "_impl.cc.t")

    os.rename(source_impl, target_impl)
    with open(target_impl, "r") as f:
        content = f.readlines()

    with open(target_impl, "w") as f:
        new_content = []
        instantiated = False
        for line in content:
            line = line.replace("@GUARD_NAME_IMPL@",
                                "_".join([real_name, "impl_h"]).upper())
            if "typedef" in line or "class" in line:
                line = line.replace("@NAME@", " " + real_name + "<T> ")
                line = line.replace("@BASE_NAME@", " " + real_name + "<T> ")
            else:
                line = line.replace("@NAME@", real_name)
                line = line.replace("@BASE_NAME@", real_name)
            line = line.replace("@IMPL_NAME@", real_name + "_impl<T>")
            line = line.replace("@NAME_IMPL@", real_name + "_impl<T> ")
            line = line.replace("@I_TYPE@", "T")
            line = line.replace("@O_TYPE@", "T")
            line = line.replace("@TYPE@", "T")
            if "@WARNING@" in line:
                continue
            if "class" in line:
                new_content.append("template<class T>\n")
            if not instantiated and "} /* namespace" in line:
                for t in block_definition.get("types"):
                    new_content.append("template class " + real_name + "<" +
                                       types[t[0]] + ">;\n")
                    instantiated = True
            new_content.append(line)
        f.writelines(new_content)


def convert_impl(block_definition):
    """
    Convert the impl header and implementation
    """
    convert_impl_header(block_definition)
    convert_impl_impl(block_definition)


def handle_template_conversion(block_definition):
    """
    Convert gengen templates to C++ templates for simple cases
    which only have one type for input and output
    """
    if block_definition.get("single_type", None) == True:
        if block_definition.get("template_type") == "GR_EXPAND_X_H":
            convert_public_header(block_definition)
        elif block_definition.get(
                "template_type"
        ) == "GR_EXPAND_X_CC_H_IMPL" or block_definition.get(
                "template_type") == "GR_EXPAND_X_CC_H":
            convert_impl(block_definition)
        rewrite_cmakelists(block_definition)


def find_template_blocks(cmake_file):
    """
    Match every line in a CMakeLists.txt file with a template regex
    """
    blocks = []
    with open(cmake_file, "r") as f:
        for line in f:
            result = re.match(template_regex, line)
            if result is not None:
                r = result.groupdict()
                r["types"] = r.get("types", "").split(" ")
                if all([(t[1:] == t[:-1] or len(t) == 1) for t in r["types"]]):
                    r["single_type"] = True
                else:
                    r["single_type"] = False
                blocks.append(r)
    return blocks


def walk_and_find(root):
    """
    Identify templated blocks by looking in the CMakeLists
    """
    all_blocks = []
    for (dirpath, dirnames, filenames) in os.walk(root):
        if "CMakeLists.txt" in filenames:
            blocks = find_template_blocks(
                os.path.join(dirpath, "CMakeLists.txt"))
            all_blocks.extend([{**block, "path": dirpath} for block in blocks])
    return all_blocks


def parse_args():
    parser = ArgumentParser()
    parser.add_argument(
        '--directory_root',
        "-d",
        help="Root directory to start search",
        default=os.environ.get("PWD"))
    parser.add_argument(
        "--no_changes",
        "-n",
        dest="no_changes",
        default=False,
        action="store_true",
        help="Only show found templated blocks, don't apply changes")
    return parser.parse_args()


def main():
    """
    Run this if the program was invoked on the commandline
    """
    args = parse_args()
    all_blocks = walk_and_find(args.directory_root)
    for block in all_blocks:
        if not args.no_changes:
            handle_template_conversion(block)
        else:
            print(block)
    return True


if __name__ == "__main__":
    exit(not (main()))



#!/usr/bin/env python
#
# Copyright 2014 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


import numpy
import copy

from gnuradio import gr, gr_unittest, blocks
from gnuradio import fec


class test_ber_bf(gr_unittest.TestCase):
    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_000(self):
        # Cause a single bit error out of 8*N bits
        # using streaming mode

        mode = False
        N = 10000
        data0 = numpy.random.randint(0, 256, N).tolist()
        data1 = copy.deepcopy(data0)
        data1[0] ^= 0x01

        src0 = blocks.vector_source_b(data0)
        src1 = blocks.vector_source_b(data1)
        op = fec.ber_bf(mode)
        dst = blocks.vector_sink_f()

        self.tb.connect(src0, (op, 0))
        self.tb.connect(src1, (op, 1))
        self.tb.connect(op, dst)
        self.tb.run()

        data = dst.data()
        # [numpy.log10(1.0 / (8.0 * N)), ]
        expected_result = self.log_ber(1., N)

        self.assertFloatTuplesAlmostEqual(expected_result, data, 5)

    def test_001(self):
        # Cause a single bit error out of 8*N bits
        # using test mode

        mode = True
        N = 1000
        data0 = numpy.random.randint(0, 256, N).tolist()
        data1 = copy.deepcopy(data0)
        data1[0] ^= 0x01

        src0 = blocks.vector_source_b(data0)
        src1 = blocks.vector_source_b(data1)
        op = fec.ber_bf(mode, 1)
        dst = blocks.vector_sink_f()

        self.tb.connect(src0, (op, 0))
        self.tb.connect(src1, (op, 1))
        self.tb.connect(op, dst)
        self.tb.run()

        data = dst.data()
        expected_result = self.log_ber(1., N)

        self.assertFloatTuplesAlmostEqual(expected_result, data, 5)

    def test_002(self):
        # Cause 8 bit errors out of 8*N bits
        # using test mode

        mode = True
        N = 1000
        data0 = numpy.random.randint(0, 256, N).tolist()
        data1 = copy.deepcopy(data0)
        data1[0] ^= 0xFF

        src0 = blocks.vector_source_b(data0)
        src1 = blocks.vector_source_b(data1)
        op = fec.ber_bf(mode, 1, -2.0)
        dst = blocks.vector_sink_f()

        self.tb.connect(src0, (op, 0))
        self.tb.connect(src1, (op, 1))
        self.tb.connect(op, dst)
        self.tb.run()

        data = dst.data()
        expected_result = self.log_ber(8., N)

        self.assertFloatTuplesAlmostEqual(expected_result, data, 5)

    def test_003(self):
        # Cause a 8 bit errors out of 8*N bits
        # using test mode
        # Exit if BER < -2.0

        mode = True
        N = 1000
        data0 = numpy.random.randint(0, 256, N).tolist()
        data1 = copy.deepcopy(data0)
        data1[0] ^= 0xFF

        src0 = blocks.vector_source_b(data0)
        src1 = blocks.vector_source_b(data1)
        op = fec.ber_bf(mode, 10, -2.0)
        dst = blocks.vector_sink_f()

        self.tb.connect(src0, (op, 0))
        self.tb.connect(src1, (op, 1))
        self.tb.connect(op, dst)
        self.tb.run()

        data = dst.data()
        expected_result = [-2.0, ]

        print(data)
        print(expected_result)

        self.assertFloatTuplesAlmostEqual(expected_result, data, 5)

    def test_004(self):
        # Cause 16 consecutive bit errors out of 8*N bits
        # make sure bytes are only read once.
        # using streaming mode

        mode = False
        N = 10000
        data0 = numpy.random.randint(0, 256, N).tolist()
        data1 = copy.deepcopy(data0)
        data1[0] ^= 0xFF
        data1[1] ^= 0xFF

        src0 = blocks.vector_source_b(data0)
        src1 = blocks.vector_source_b(data1)
        op = fec.ber_bf(mode)
        dst = blocks.vector_sink_f()

        self.tb.connect(src0, (op, 0))
        self.tb.connect(src1, (op, 1))
        self.tb.connect(op, dst)
        self.tb.run()

        data = dst.data()
        expected_result = self.log_ber(16, N)

        self.assertFloatTuplesAlmostEqual(expected_result, data, 5)

    def log_ber(self, n_errors, N):
        return numpy.log10(1. * n_errors / (8.0 * N)),


if __name__ == '__main__':
    gr_unittest.run(test_ber_bf)



#!/usr/bin/env python
#
# Copyright 2004,2007,2010,2012-2014 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, blocks


class test_interleave (gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_int_001(self):
        lenx = 64
        src0 = blocks.vector_source_f(list(range(0, lenx, 4)))
        src1 = blocks.vector_source_f(list(range(1, lenx, 4)))
        src2 = blocks.vector_source_f(list(range(2, lenx, 4)))
        src3 = blocks.vector_source_f(list(range(3, lenx, 4)))
        op = blocks.interleave(gr.sizeof_float)
        dst = blocks.vector_sink_f()

        self.tb.connect(src0, (op, 0))
        self.tb.connect(src1, (op, 1))
        self.tb.connect(src2, (op, 2))
        self.tb.connect(src3, (op, 3))
        self.tb.connect(op, dst)
        self.tb.run()
        expected_result = tuple(range(lenx))
        result_data = dst.data()
        self.assertFloatTuplesAlmostEqual(expected_result, result_data)

    def test_int_002(self):
        blksize = 4
        lenx = 64
        def plusup_big(a): return a + (blksize * 4)
        def plusup_little(a): return a + blksize
        a_vec = list(range(0, blksize))
        for i in range(0, (lenx // (4 * blksize)) - 1):
            a_vec += list(map(plusup_big, a_vec[len(a_vec) - blksize:]))

        b_vec = list(map(plusup_little, a_vec))
        c_vec = list(map(plusup_little, b_vec))
        d_vec = list(map(plusup_little, c_vec))

        src0 = blocks.vector_source_f(a_vec)
        src1 = blocks.vector_source_f(b_vec)
        src2 = blocks.vector_source_f(c_vec)
        src3 = blocks.vector_source_f(d_vec)
        op = blocks.interleave(gr.sizeof_float, blksize)
        dst = blocks.vector_sink_f()

        self.tb.connect(src0, (op, 0))
        self.tb.connect(src1, (op, 1))
        self.tb.connect(src2, (op, 2))
        self.tb.connect(src3, (op, 3))
        self.tb.connect(op, dst)
        self.tb.run()
        expected_result = tuple(range(lenx))
        result_data = dst.data()

        self.assertFloatTuplesAlmostEqual(expected_result, result_data)

    def test_deint_001(self):
        lenx = 64
        src = blocks.vector_source_f(list(range(lenx)))
        op = blocks.deinterleave(gr.sizeof_float)
        dst0 = blocks.vector_sink_f()
        dst1 = blocks.vector_sink_f()
        dst2 = blocks.vector_sink_f()
        dst3 = blocks.vector_sink_f()

        self.tb.connect(src, op)
        self.tb.connect((op, 0), dst0)
        self.tb.connect((op, 1), dst1)
        self.tb.connect((op, 2), dst2)
        self.tb.connect((op, 3), dst3)
        self.tb.run()

        expected_result0 = tuple(range(0, lenx, 4))
        expected_result1 = tuple(range(1, lenx, 4))
        expected_result2 = tuple(range(2, lenx, 4))
        expected_result3 = tuple(range(3, lenx, 4))

        self.assertFloatTuplesAlmostEqual(expected_result0, dst0.data())
        self.assertFloatTuplesAlmostEqual(expected_result1, dst1.data())
        self.assertFloatTuplesAlmostEqual(expected_result2, dst2.data())
        self.assertFloatTuplesAlmostEqual(expected_result3, dst3.data())

    def test_deint_002(self):
        blksize = 4
        lenx = 64
        src = blocks.vector_source_f(list(range(lenx)))
        op = blocks.deinterleave(gr.sizeof_float, blksize)
        dst0 = blocks.vector_sink_f()
        dst1 = blocks.vector_sink_f()
        dst2 = blocks.vector_sink_f()
        dst3 = blocks.vector_sink_f()

        self.tb.connect(src, op)
        self.tb.connect((op, 0), dst0)
        self.tb.connect((op, 1), dst1)
        self.tb.connect((op, 2), dst2)
        self.tb.connect((op, 3), dst3)
        self.tb.run()

        def plusup_big(a): return a + (blksize * 4)
        def plusup_little(a): return a + blksize
        a_vec = list(range(0, blksize))
        for i in range(0, (lenx // (4 * blksize)) - 1):
            a_vec += list(map(plusup_big, a_vec[len(a_vec) - blksize:]))

        b_vec = list(map(plusup_little, a_vec))
        c_vec = list(map(plusup_little, b_vec))
        d_vec = list(map(plusup_little, c_vec))

        expected_result0 = tuple(a_vec)
        expected_result1 = tuple(b_vec)
        expected_result2 = tuple(c_vec)
        expected_result3 = tuple(d_vec)

        self.assertFloatTuplesAlmostEqual(expected_result0, dst0.data())
        self.assertFloatTuplesAlmostEqual(expected_result1, dst1.data())
        self.assertFloatTuplesAlmostEqual(expected_result2, dst2.data())
        self.assertFloatTuplesAlmostEqual(expected_result3, dst3.data())


if __name__ == '__main__':
    gr_unittest.run(test_interleave)



#!/usr/bin/env python
#
# Copyright 2006,2007,2010,2011,2013,2017 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, digital, blocks

default_access_code = '\xAC\xDD\xA4\xE2\xF2\x8C\x20\xFC'


def string_to_1_0_list(s):
    r = []
    for ch in s:
        x = ord(ch)
        for i in range(8):
            t = (x >> i) & 0x1
            r.append(t)

    return r


def to_1_0_string(L):
    return ''.join([chr(x + ord('0')) for x in L])


class test_correlate_access_code(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_001(self):
        pad = [0, ] * 64
        #           0  0  0  1  0  0  0  1
        src_data = [1, 0, 1, 1, 1, 1, 0, 1, 1] + pad + [0, ] * 7
        expected_result = pad + [1, 0, 1, 1, 3, 1, 0, 1, 1, 2] + [0, ] * 6
        src = blocks.vector_source_b(src_data)
        op = digital.correlate_access_code_bb("1011", 0)
        dst = blocks.vector_sink_b()
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertEqual(expected_result, result_data)

    def test_002(self):
        code = list(string_to_1_0_list(default_access_code))
        access_code = to_1_0_string(code)
        pad = [0, ] * 64
        # print code
        # print access_code
        src_data = code + [1, 0, 1, 1] + pad
        expected_result = pad + code + [3, 0, 1, 1]
        src = blocks.vector_source_b(src_data)
        op = digital.correlate_access_code_bb(access_code, 0)
        dst = blocks.vector_sink_b()
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertEqual(expected_result, result_data)

    def test_003(self):
        code = list(string_to_1_0_list(default_access_code))
        access_code = to_1_0_string(code)
        pad = [0, ] * 64
        # print code
        # print access_code
        src_data = code + [1, 0, 1, 1] + pad
        expected_result = code + [1, 0, 1, 1] + pad
        src = blocks.vector_source_b(src_data)
        op = digital.correlate_access_code_tag_bb(access_code, 0, "test")
        dst = blocks.vector_sink_b()
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertEqual(expected_result, result_data)

    def test_004(self):
        code = list(string_to_1_0_list(default_access_code))
        access_code = to_1_0_string(code)
        pad = [0, ] * 64
        # print code
        # print access_code
        src_bits = code + [1, 0, 1, 1] + pad
        src_data = [2.0 * x - 1.0 for x in src_bits]
        expected_result_bits = code + [1, 0, 1, 1] + pad
        expected_result = [2.0 * x - 1.0 for x in expected_result_bits]
        src = blocks.vector_source_f(src_data)
        op = digital.correlate_access_code_tag_ff(access_code, 0, "test")
        dst = blocks.vector_sink_f()
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertFloatTuplesAlmostEqual(expected_result, result_data, 5)


if __name__ == '__main__':
    gr_unittest.run(test_correlate_access_code)



#!/usr/bin/env python
# Copyright 2012 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#

import random

from gnuradio import gr, gr_unittest, blocks, digital
from gnuradio.digital.utils import tagged_streams
import pmt


class qa_packet_headerparser_b (gr_unittest.TestCase):

    def setUp(self):
        random.seed(0)
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_001_t(self):
        """
        First header: Packet length 4, packet num 0
        Second header: Packet 2, packet num 1
        Third header: Invalid (CRC does not check) (would be len 4, num 2)
        """
        encoded_headers = (
            #   | Number of bytes                    | Packet number                      | CRC
            0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 0, 0, 1,
            0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1,
            0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1
        )
        packet_len_tagname = "packet_len"
        random_tag = gr.tag_t()
        random_tag.offset = 5
        random_tag.key = pmt.string_to_symbol("foo")
        random_tag.value = pmt.from_long(42)
        src = blocks.vector_source_b(encoded_headers, tags=(random_tag,))
        parser = digital.packet_headerparser_b(32, packet_len_tagname)
        sink = blocks.message_debug()
        self.tb.connect(src, parser)
        self.tb.msg_connect(parser, "header_data", sink, "store")
        self.tb.start()
        self.waitFor(lambda: sink.num_messages() == 3)
        self.tb.stop()
        self.tb.wait()
        self.assertEqual(sink.num_messages(), 3)
        msg1 = pmt.to_python(sink.get_message(0))
        msg2 = pmt.to_python(sink.get_message(1))
        msg3 = pmt.to_python(sink.get_message(2))
        self.assertEqual(msg1, {'packet_len': 4, 'packet_num': 0, 'foo': 42})
        self.assertEqual(msg2, {'packet_len': 2, 'packet_num': 1})
        self.assertEqual(msg3, False)

    def test_002_pipe(self):
        """
        Create N packets of random length, pipe them through header generator,
        back to header parser, make sure output is the same.
        """
        N = 20
        header_len = 32
        packet_len_tagname = "packet_len"
        packet_lengths = [random.randint(1, 100) for x in range(N)]
        data, tags = tagged_streams.packets_to_vectors(
            [list(range(packet_lengths[i])) for i in range(N)], packet_len_tagname)
        src = blocks.vector_source_b(data, False, 1, tags)
        header_gen = digital.packet_headergenerator_bb(
            header_len, packet_len_tagname)
        header_parser = digital.packet_headerparser_b(
            header_len, packet_len_tagname)
        sink = blocks.message_debug()
        self.tb.connect(src, header_gen, header_parser)
        self.tb.msg_connect(header_parser, "header_data", sink, "store")
        self.tb.start()
        self.waitFor(lambda: sink.num_messages() == N)
        self.tb.stop()
        self.tb.wait()
        self.assertEqual(sink.num_messages(), N)
        for i in range(N):
            msg = pmt.to_python(sink.get_message(i))
            self.assertEqual(
                msg, {
                    'packet_len': packet_lengths[i], 'packet_num': i})

    def test_003_ofdm(self):
        """ Header 1: 193 bytes
        Header 2: 8 bytes
        2 bits per complex symbol, 32 carriers => 64 bits = 8 bytes per OFDM symbol
                                    4 carriers =>  8 bits = 1 byte  per OFDM symbol
                                    8 carriers => 16 bits = 2 bytes per OFDM symbol
        Means we need 52 carriers to store the 193 bytes.
        """
        encoded_headers = (
            #   | Number of bytes                    | Packet number                      | CRC
            1, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 1, 0, 1, 1, 0,
            0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0,
        )
        packet_len_tagname = "packet_len"
        frame_len_tagname = "frame_len"
        src = blocks.vector_source_b(encoded_headers)
        header_formatter = digital.packet_header_ofdm(
            # 32/4/8 carriers are occupied (which doesn't matter here)
            (list(range(32)), list(range(4)), list(range(8))),
            1,         # 1 OFDM symbol per header (= 32 bits)
            packet_len_tagname,
            frame_len_tagname,
            "packet_num",
            1,         # 1 bit per header symbols (BPSK)
            2          # 2 bits per payload symbol (QPSK)
        )
        parser = digital.packet_headerparser_b(header_formatter.base())
        sink = blocks.message_debug()
        self.tb.connect(src, parser)
        self.tb.msg_connect(parser, "header_data", sink, "store")
        self.tb.start()
        self.waitFor(lambda: sink.num_messages() == 2)
        self.tb.stop()
        self.tb.wait()
        self.assertEqual(sink.num_messages(), 2)
        msg1 = pmt.to_python(sink.get_message(0))
        msg2 = pmt.to_python(sink.get_message(1))
        # Multiply with 4 because unpacked bytes have only two bits
        self.assertEqual(msg1, {'packet_len': 193 * 4,
                                'frame_len': 52, 'packet_num': 0})
        self.assertEqual(
            msg2, {
                'packet_len': 8 * 4, 'frame_len': 1, 'packet_num': 1})

    def test_004_ofdm_scramble(self):
        """
        Test scrambling for OFDM header gen
        """
        header_len = 32
        packet_length = 23
        packet_len_tagname = "packet_len"
        frame_len_tagname = "frame_len"
        data, tags = tagged_streams.packets_to_vectors(
            [list(range(packet_length)), list(range(packet_length)), ], packet_len_tagname)
        src = blocks.vector_source_b(data, False, 1, tags)
        header_formatter = digital.packet_header_ofdm(
            # 32 carriers are occupied (which doesn't matter here)
            (list(range(32)),),
            1,         # 1 OFDM symbol per header (= 32 bits)
            packet_len_tagname,
            frame_len_tagname,
            "packet_num",
            1,         # 1 bit per header symbols (BPSK)
            2,          # 2 bits per payload symbol (QPSK)
            scramble_header=True
        )
        header_gen = digital.packet_headergenerator_bb(header_formatter.base())
        header_parser = digital.packet_headerparser_b(header_formatter.base())
        sink = blocks.message_debug()
        self.tb.connect(src, header_gen, header_parser)
        self.tb.msg_connect(header_parser, "header_data", sink, "store")
        self.tb.start()
        self.waitFor(lambda: sink.num_messages() == 2)
        self.tb.stop()
        self.tb.wait()
        msg = pmt.to_python(sink.get_message(0))
        self.assertEqual(
            msg, {'packet_len': packet_length, 'packet_num': 0, 'frame_len': 4})
        msg = pmt.to_python(sink.get_message(1))
        self.assertEqual(
            msg, {'packet_len': packet_length, 'packet_num': 1, 'frame_len': 4})


if __name__ == '__main__':
    gr_unittest.run(qa_packet_headerparser_b)



#!/usr/bin/env python
#
# Copyright 2012,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, blocks


class test_conjugate (gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_000(self):
        src_data = [-2 - 2j, -1 - 1j, -2 + 2j, -1 + 1j,
                    2 - 2j, 1 - 1j, 2 + 2j, 1 + 1j,
                    0 + 0j]

        exp_data = [-2 + 2j, -1 + 1j, -2 - 2j, -1 - 1j,
                    2 + 2j, 1 + 1j, 2 - 2j, 1 - 1j,
                    0 - 0j]

        src = blocks.vector_source_c(src_data)
        op = blocks.conjugate_cc()
        dst = blocks.vector_sink_c()

        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertEqual(exp_data, result_data)


if __name__ == '__main__':
    gr_unittest.run(test_conjugate)



#!/usr/bin/env python
#
# Copyright 2012,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


import math
import random
from gnuradio import gr, gr_unittest, analog, blocks


class test_ctcss_squelch(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_ctcss_squelch_001(self):
        # Test set/gets

        rate = 1
        rate2 = 2
        freq = 100
        level = 0.5
        length = 1
        ramp = 1
        ramp2 = 2
        gate = True
        gate2 = False

        op = analog.ctcss_squelch_ff(rate, freq, level,
                                     length, ramp, gate)

        op.set_ramp(ramp2)
        r = op.ramp()
        self.assertEqual(ramp2, r)

        op.set_gate(gate2)
        g = op.gate()
        self.assertEqual(gate2, g)

    def test_ctcss_squelch_002(self):
        # Test runtime, gate=True
        rate = 8000
        freq = 100
        other_freq = 103.5
        level = 0.01
        length = 0
        ramp = 0
        gate = True

        random.seed(1)
        src_data = [0.5 * math.sin(2 * math.pi * 1000 * x / rate) + random.gauss(0, 0.1) for x in range(rate)]

        # First half-second has incorrect CTCSS tone
        for x in range(0, int(rate * 0.500)):
            src_data[x] += 0.15 * math.sin(2 * math.pi * other_freq * x / rate)

        # Second half-second has correct CTCSS tone
        for x in range(int(rate * 0.500), rate):
            src_data[x] += 0.15 * math.sin(2 * math.pi * freq * x / rate)

        src = blocks.vector_source_f(src_data)
        op = analog.ctcss_squelch_ff(rate, freq, level,
                                     length, ramp, gate)
        dst = blocks.vector_sink_f()

        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()

        result_data = dst.data()

        # Squelch should open ~100 ms after the correct CTCSS tone appears
        # so ~400 ms of audio should make it past the gate
        self.assertGreater(len(result_data), rate * 0.390)
        self.assertLess(len(result_data), rate * 0.410)
        self.assertFloatTuplesAlmostEqual(src_data[-len(result_data):], result_data, 6)

    def test_ctcss_squelch_003(self):
        # Test runtime, gate=False
        rate = 8000
        freq = 100
        other_freq = 103.5
        level = 0.01
        length = 0
        ramp = 0
        gate = False

        random.seed(1)
        src_data = [0.5 * math.sin(2 * math.pi * 1000 * x / rate) + random.gauss(0, 0.1) for x in range(rate)]

        # First half-second has incorrect CTCSS tone
        for x in range(0, rate // 2):
            src_data[x] += 0.15 * math.sin(2 * math.pi * other_freq * x / rate)

        # Second half-second has correct CTCSS tone
        for x in range(rate // 2, rate):
            src_data[x] += 0.15 * math.sin(2 * math.pi * freq * x / rate)

        src = blocks.vector_source_f(src_data)
        op = analog.ctcss_squelch_ff(rate, freq, level,
                                     length, ramp, gate)
        dst = blocks.vector_sink_f()

        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()

        result_data = dst.data()

        # Squelch should open ~100 ms after the correct CTCSS tone appears
        min_zero_samples = int(rate * 0.590)
        self.assertFloatTuplesAlmostEqual([0] * min_zero_samples, result_data[:min_zero_samples], 6)
        max_zero_samples = int(rate * 0.610)
        self.assertFloatTuplesAlmostEqual(src_data[max_zero_samples:], result_data[max_zero_samples:], 6)


if __name__ == '__main__':
    gr_unittest.run(test_ctcss_squelch)



#!/usr/bin/env python
#
# Copyright 2012,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, analog, blocks


def clip(x, lo, hi):
    if(x < lo):
        return lo
    elif(x > hi):
        return hi
    else:
        return x


class test_rail(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_rail_001(self):
        # Test set/gets

        hi1 = 1
        hi2 = 2
        lo1 = -1
        lo2 = -2

        op = analog.rail_ff(lo1, hi1)

        op.set_hi(hi2)
        h = op.hi()
        self.assertEqual(hi2, h)

        op.set_lo(lo2)
        l = op.lo()
        self.assertEqual(lo2, l)

    def test_rail_002(self):
        lo = -0.75
        hi = 0.90
        src_data = [-2, -1, -0.5, -0.25, 0, 0.25, 0.5, 1, 2]
        expected_result = [clip(x, lo, hi) for x in src_data]

        src = blocks.vector_source_f(src_data)
        op = analog.rail_ff(lo, hi)
        dst = blocks.vector_sink_f()

        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()

        result_data = dst.data()
        self.assertFloatTuplesAlmostEqual(expected_result, result_data, 4)


if __name__ == '__main__':
    gr_unittest.run(test_rail)



import pytest

from grc.core.blocks._templates import MakoTemplates
from grc.core.errors import TemplateError


class Block(object):
    namespace_templates = {}

    templates = MakoTemplates(None)

    def __init__(self, **kwargs):
        self.namespace_templates.update(kwargs)


def test_simple():
    t = MakoTemplates(_bind_to=Block(num='123'), test='abc${num}')
    assert t['test'] == 'abc${num}'
    assert t.render('test') == 'abc123'
    assert 'abc${num}' in t._template_cache


def test_instance():
    block = Block(num='123')
    block.templates['test'] = 'abc${num}'
    assert block.templates.render('test') == 'abc123'
    assert block.templates is block.__dict__['templates']


def test_list():
    templates = ['abc${num}', '${2 * num}c']
    t = MakoTemplates(_bind_to=Block(num='123'), test=templates)
    assert t['test'] == templates
    assert t.render('test') == ['abc123', '123123c']
    assert set(templates) == set(t._template_cache.keys())


def test_parse_error():
    with pytest.raises(TemplateError):
        MakoTemplates(_bind_to=Block(num='123'),
                      test='abc${num NOT CLOSING').render('test')


def test_parse_error2():
    with pytest.raises(TemplateError):
        MakoTemplates(_bind_to=Block(num='123'),
                      test='abc${ WRONG_VAR }').render('test')



#!/usr/bin/env python
#
# Copyright 2007,2010,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, blocks


class test_peak_detector2(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_peak1(self):
        # print "\n\nTEST 1"
        tb = self.tb

        n = 10
        data = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10,
                9, 8, 7, 6, 5, 4, 3, 2, 1, 0, ] + n * [0, ]

        expected_result = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1,
                           0, 0, 0, 0, 0, 0, 0, 0, 0, 0] + n * [0, ]

        src = blocks.vector_source_f(data, False)
        regen = blocks.peak_detector2_fb(7.0, 25, 0.001)
        dst = blocks.vector_sink_b()

        tb.connect(src, regen)
        tb.connect(regen, dst)
        tb.run()

        dst_data = dst.data()

        self.assertEqual(expected_result, dst_data)

    def test_peak2(self):
        # print "\n\nTEST 2"
        tb = self.tb

        n = 10
        data = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10,
                9, 8, 7, 6, 5, 4, 3, 2, 1, 0, ] + n * [0, ]

        expected_result = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1,
                           0, 0, 0, 0, 0, 0, 0, 0, 0, 0] + n * [0, ]

        src = blocks.vector_source_f(data, False)
        regen = blocks.peak_detector2_fb(
            7.0, 1000, 0.001)  # called with a LONG window
        dst = blocks.vector_sink_b()

        tb.connect(src, regen)
        tb.connect(regen, dst)
        tb.run()

        dst_data = dst.data()

        # here we know that the block will terminate prematurely, so we compare
        # only part of the expected_result
        self.assertEqual(expected_result[0:len(dst_data)], dst_data)

    def test_peak3(self):
        # print "\n\nTEST 3"
        tb = self.tb

        l = 8100
        m = 100
        n = 10
        data = l * [0, ] + [10, ] + m * [0, ] + [100, ] + n * [0, ]
        expected_result = l * [0, ] + [0, ] + m * [0, ] + [1, ] + n * [0, ]

        src = blocks.vector_source_f(data, False)
        regen = blocks.peak_detector2_fb(7.0, 105, 0.001)
        dst = blocks.vector_sink_b()

        tb.connect(src, regen)
        tb.connect(regen, dst)
        tb.run()

        dst_data = dst.data()

        self.assertEqual(expected_result, dst_data)

    def test_peak4(self):
        # print "\n\nTEST 4"
        tb = self.tb

        l = 8100
        m = 100
        n = 10
        data = l * [0, ] + [10, ] + m * [0, ] + [100, ] + n * [0, ]
        expected_result = l * [0, ] + [0, ] + m * [0, ] + [1, ] + n * [0, ]

        src = blocks.vector_source_f(data, False)
        regen = blocks.peak_detector2_fb(7.0, 150, 0.001)
        dst = blocks.vector_sink_b()

        tb.connect(src, regen)
        tb.connect(regen, dst)
        tb.run()

        dst_data = dst.data()

        # here we know that the block will terminate prematurely, so we compare
        # only part of the expected_result
        self.assertEqual(expected_result[0:len(dst_data)], dst_data)

    def test_peak5(self):
        # print "\n\nTEST 5"
        tb = self.tb

        data = [0, 0, 0, 10, 0, 0, 0, 0]
        alpha = 0.25
        expected_result_peak = [0, 0, 0, 1, 0, 0, 0, 0]
        expected_result_average = [0]
        for i in data:
            expected_result_average.append(
                expected_result_average[-1] * (1 - alpha) + i * alpha)

        src = blocks.vector_source_f(data, False)
        regen = blocks.peak_detector2_fb(2.0, 2, alpha)
        dst = blocks.vector_sink_b()
        avg = blocks.vector_sink_f()

        tb.connect(src, regen)
        tb.connect((regen, 0), dst)
        tb.connect((regen, 1), avg)
        tb.run()

        dst_data = dst.data()
        dst_avg = avg.data()

        self.assertEqual(expected_result_peak, dst_data)
        self.assertFloatTuplesAlmostEqual(expected_result_average[1:], dst_avg)


if __name__ == '__main__':
    gr_unittest.run(test_peak_detector2)



#!/usr/bin/env python
#
# Copyright 2008,2010,2012,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, filter, blocks


def fir_filter(x, taps, decim=1):
    y = []
    x2 = (len(taps) - 1) * [0, ] + x
    for i in range(0, len(x), decim):
        yi = 0
        for j in range(len(taps)):
            yi += taps[len(taps) - 1 - j] * x2[i + j]
        y.append(yi)
    return y


class test_filter(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_fir_filter_fff_001(self):
        decim = 1
        taps = 20 * [0.5, 0.5]
        src_data = 40 * [1, 2, 3, 4]
        expected_data = fir_filter(src_data, taps, decim)

        src = blocks.vector_source_f(src_data)
        op = filter.fir_filter_fff(decim, taps)
        dst = blocks.vector_sink_f()
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertFloatTuplesAlmostEqual(expected_data, result_data, 5)

    def test_fir_filter_fff_002(self):
        decim = 4
        taps = 20 * [0.5, 0.5]
        src_data = 40 * [1, 2, 3, 4]
        expected_data = fir_filter(src_data, taps, decim)

        src = blocks.vector_source_f(src_data)
        op = filter.fir_filter_fff(decim, taps)
        dst = blocks.vector_sink_f()
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertFloatTuplesAlmostEqual(expected_data, result_data, 5)

    def test_fir_filter_ccf_001(self):
        decim = 1
        taps = 20 * [0.5, 0.5]
        src_data = 40 * [1 + 1j, 2 + 2j, 3 + 3j, 4 + 4j]
        expected_data = fir_filter(src_data, taps, decim)

        src = blocks.vector_source_c(src_data)
        op = filter.fir_filter_ccf(decim, taps)
        dst = blocks.vector_sink_c()
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertComplexTuplesAlmostEqual(expected_data, result_data, 5)

    def test_fir_filter_ccf_002(self):
        decim = 4
        taps = 20 * [0.5, 0.5]
        src_data = 40 * [1 + 1j, 2 + 2j, 3 + 3j, 4 + 4j]
        expected_data = fir_filter(src_data, taps, decim)

        src = blocks.vector_source_c(src_data)
        op = filter.fir_filter_ccf(decim, taps)
        dst = blocks.vector_sink_c()
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertComplexTuplesAlmostEqual(expected_data, result_data, 5)

    def test_fir_filter_ccc_001(self):
        decim = 1
        taps = 20 * [0.5 + 1j, 0.5 + 1j]
        src_data = 40 * [1 + 1j, 2 + 2j, 3 + 3j, 4 + 4j]
        expected_data = fir_filter(src_data, taps, decim)

        src = blocks.vector_source_c(src_data)
        op = filter.fir_filter_ccc(decim, taps)
        dst = blocks.vector_sink_c()
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertComplexTuplesAlmostEqual(expected_data, result_data, 5)

    def test_fir_filter_ccc_002(self):
        decim = 1
        taps = filter.firdes.low_pass(1, 1, 0.1, 0.01)
        src_data = 10 * [1 + 1j, 2 + 2j, 3 + 3j, 4 + 4j]
        expected_data = fir_filter(src_data, taps, decim)

        src = blocks.vector_source_c(src_data)
        op = filter.fir_filter_ccc(decim, taps)
        dst = blocks.vector_sink_c()
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertComplexTuplesAlmostEqual(expected_data, result_data, 5)

    def test_fir_filter_ccc_003(self):
        decim = 4
        taps = 20 * [0.5 + 1j, 0.5 + 1j]
        src_data = 40 * [1 + 1j, 2 + 2j, 3 + 3j, 4 + 4j]
        expected_data = fir_filter(src_data, taps, decim)

        src = blocks.vector_source_c(src_data)
        op = filter.fir_filter_ccc(decim, taps)
        dst = blocks.vector_sink_c()
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertComplexTuplesAlmostEqual(expected_data, result_data, 5)

    def test_fir_filter_scc_001(self):
        decim = 1
        taps = 20 * [0.5 + 1j, 0.5 + 1j]
        src_data = 40 * [1, 2, 3, 4]
        expected_data = fir_filter(src_data, taps, decim)

        src = blocks.vector_source_s(src_data)
        op = filter.fir_filter_scc(decim, taps)
        dst = blocks.vector_sink_c()
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertComplexTuplesAlmostEqual(expected_data, result_data, 5)

    def test_fir_filter_scc_002(self):
        decim = 4
        taps = 20 * [0.5 + 1j, 0.5 + 1j]
        src_data = 40 * [1, 2, 3, 4]
        expected_data = fir_filter(src_data, taps, decim)

        src = blocks.vector_source_s(src_data)
        op = filter.fir_filter_scc(decim, taps)
        dst = blocks.vector_sink_c()
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertComplexTuplesAlmostEqual(expected_data, result_data, 5)

    def test_fir_filter_fsf_001(self):
        decim = 1
        taps = 20 * [0.5, 0.5]
        src_data = 40 * [1, 2, 3, 4]
        expected_data = fir_filter(src_data, taps, decim)
        expected_data = [round(e) for e in expected_data]

        src = blocks.vector_source_f(src_data)
        op = filter.fir_filter_fsf(decim, taps)
        dst = blocks.vector_sink_s()
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.data()

        # TODO: Remove fallback once GNU Radio's required VOLK_VERSION is >= 3.1.0
        try:
            self.assertEqual(expected_data, result_data)
        except AssertionError:
            expected_data_distorted = [int(e) for e in fir_filter(src_data, taps, decim)]
            self.assertEqual(expected_data_distorted, result_data)

    def test_fir_filter_fsf_002(self):
        decim = 4
        taps = 20 * [0.5, 0.5]
        src_data = 40 * [1, 2, 3, 4]
        expected_data = fir_filter(src_data, taps, decim)
        expected_data = [round(e) for e in expected_data]

        src = blocks.vector_source_f(src_data)
        op = filter.fir_filter_fsf(decim, taps)
        dst = blocks.vector_sink_s()
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.data()

        # TODO: Remove fallback once GNU Radio's required VOLK_VERSION is >= 3.1.0
        try:
            self.assertEqual(expected_data, result_data)
        except AssertionError:
            expected_data_distorted = [int(e) for e in fir_filter(src_data, taps, decim)]
            self.assertEqual(expected_data_distorted, result_data)


if __name__ == '__main__':
    gr_unittest.run(test_filter)



#!/usr/bin/env python
#
# Copyright 2013-2014 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, digital, blocks
from math import sqrt
from numpy import random, vectorize


class test_constellation_soft_decoder(gr_unittest.TestCase):

    def setUp(self):
        random.seed(0)
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def helper_with_lut(self, prec, src_data, const_gen, const_sd_gen, decimals=5):
        cnst_pts, code = const_gen()
        Es = 1.0
        lut = digital.soft_dec_table(cnst_pts, code, prec, Es)

        constel = digital.const_normalization(cnst_pts, "POWER")
        maxamp = digital.min_max_axes(constel)

        expected_result = list()
        for s in src_data:
            res = digital.calc_soft_dec_from_table(s, lut, prec, maxamp)
            expected_result += res

        cnst = digital.constellation_calcdist(cnst_pts, code, 4, 1, digital.constellation.POWER_NORMALIZATION)
        cnst.set_soft_dec_lut(lut, int(prec))
        cnst.normalize(digital.constellation.POWER_NORMALIZATION)
        src = blocks.vector_source_c(src_data)
        op = digital.constellation_soft_decoder_cf(cnst.base())
        dst = blocks.vector_sink_f()

        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()

        actual_result = dst.data()  # fetch the contents of the sink
        # print "actual result", actual_result
        # print "expected result", expected_result
        self.assertFloatTuplesAlmostEqual(expected_result, actual_result, decimals)

    def helper_no_lut(self, prec, src_data, const_gen, const_sd_gen):
        cnst_pts, code = const_gen()
        cnst = digital.constellation_calcdist(cnst_pts, code, 2, 1)
        expected_result = list()
        for s in src_data:
            res = digital.calc_soft_dec(s, cnst.points(), code)
            expected_result += res

        src = blocks.vector_source_c(src_data)
        op = digital.constellation_soft_decoder_cf(cnst.base())
        dst = blocks.vector_sink_f()

        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()

        actual_result = dst.data()  # fetch the contents of the sink
        # print "actual result", actual_result
        # print "expected result", expected_result

        # Double vs. float precision issues between Python and C++, so
        # use only 4 decimals in comparisons.
        self.assertFloatTuplesAlmostEqual(expected_result, actual_result, 4)

    def test_constellation_soft_decoder_cf_bpsk_3(self):
        prec = 3
        src_data = (-1.0 - 1.0j, 1.0 - 1.0j, -1.0 + 1.0j, 1.0 + 1.0j,
                    -2.0 - 2.0j, 2.0 - 2.0j, -2.0 + 2.0j, 2.0 + 2.0j,
                    -0.2 - 0.2j, 0.2 - 0.2j, -0.2 + 0.2j, 0.2 + 0.2j,
                    0.3 + 0.4j, 0.1 - 1.2j, -0.8 - 0.1j, -0.4 + 0.8j,
                    0.8 + 1.0j, -0.5 + 0.1j, 0.1 + 1.2j, -1.7 - 0.9j)
        self.helper_with_lut(
            prec,
            src_data,
            digital.psk_2_0x0,
            digital.sd_psk_2_0x0)

    def test_constellation_soft_decoder_cf_bpsk_8(self):
        prec = 8
        src_data = (-1.0 - 1.0j, 1.0 - 1.0j, -1.0 + 1.0j, 1.0 + 1.0j,
                    -2.0 - 2.0j, 2.0 - 2.0j, -2.0 + 2.0j, 2.0 + 2.0j,
                    -0.2 - 0.2j, 0.2 - 0.2j, -0.2 + 0.2j, 0.2 + 0.2j,
                    0.3 + 0.4j, 0.1 - 1.2j, -0.8 - 0.1j, -0.4 + 0.8j,
                    0.8 + 1.0j, -0.5 + 0.1j, 0.1 + 1.2j, -1.7 - 0.9j)
        self.helper_with_lut(
            prec,
            src_data,
            digital.psk_2_0x0,
            digital.sd_psk_2_0x0)

    def test_constellation_soft_decoder_cf_bpsk_8_rand(self):
        prec = 8
        src_data = vectorize(complex)(
            2 * random.randn(100), 2 * random.randn(100))
        self.helper_with_lut(
            prec,
            src_data,
            digital.psk_2_0x0,
            digital.sd_psk_2_0x0)

    def test_constellation_soft_decoder_cf_bpsk_8_rand2(self):
        prec = 8
        src_data = vectorize(complex)(
            2 * random.randn(100), 2 * random.randn(100))
        self.helper_no_lut(
            prec,
            src_data,
            digital.psk_2_0x0,
            digital.sd_psk_2_0x0)

    def test_constellation_soft_decoder_cf_qpsk_3(self):
        prec = 3
        src_data = (-1.0 - 1.0j, 1.0 - 1.0j, -1.0 + 1.0j, 1.0 + 1.0j,
                    -2.0 - 2.0j, 2.0 - 2.0j, -2.0 + 2.0j, 2.0 + 2.0j,
                    -0.2 - 0.2j, 0.2 - 0.2j, -0.2 + 0.2j, 0.2 + 0.2j,
                    0.3 + 0.4j, 0.1 - 1.2j, -0.8 - 0.1j, -0.4 + 0.8j,
                    0.8 + 1.0j, -0.5 + 0.1j, 0.1 + 1.2j, -1.7 - 0.9j)
        self.helper_with_lut(
            prec,
            src_data,
            digital.psk_4_0x0_0_1,
            digital.sd_psk_4_0x0_0_1)

    def test_constellation_soft_decoder_cf_qpsk_8(self):
        prec = 8
        src_data = (-1.0 - 1.0j, 1.0 - 1.0j, -1.0 + 1.0j, 1.0 + 1.0j,
                    -2.0 - 2.0j, 2.0 - 2.0j, -2.0 + 2.0j, 2.0 + 2.0j,
                    -0.2 - 0.2j, 0.2 - 0.2j, -0.2 + 0.2j, 0.2 + 0.2j,
                    0.3 + 0.4j, 0.1 - 1.2j, -0.8 - 0.1j, -0.4 + 0.8j,
                    0.8 + 1.0j, -0.5 + 0.1j, 0.1 + 1.2j, -1.7 - 0.9j)
        self.helper_with_lut(
            prec,
            src_data,
            digital.psk_4_0x0_0_1,
            digital.sd_psk_4_0x0_0_1)

    def test_constellation_soft_decoder_cf_qpsk_8_rand(self):
        prec = 8
        src_data = vectorize(complex)(
            2 * random.randn(100), 2 * random.randn(100))
        self.helper_with_lut(
            prec,
            src_data,
            digital.psk_4_0x0_0_1,
            digital.sd_psk_4_0x0_0_1,
            3)

    def test_constellation_soft_decoder_cf_qpsk_8_rand2(self):
        prec = 8
        src_data = vectorize(complex)(
            2 * random.randn(100), 2 * random.randn(100))
        self.helper_no_lut(
            prec,
            src_data,
            digital.psk_4_0x0_0_1,
            digital.sd_psk_4_0x0_0_1)

    def test_constellation_soft_decoder_cf_qam16_3(self):
        prec = 3
        src_data = (-1.0 - 1.0j, 1.0 - 1.0j, -1.0 + 1.0j, 1.0 + 1.0j,
                    -2.0 - 2.0j, 2.0 - 2.0j, -2.0 + 2.0j, 2.0 + 2.0j,
                    -0.2 - 0.2j, 0.2 - 0.2j, -0.2 + 0.2j, 0.2 + 0.2j,
                    0.3 + 0.4j, 0.1 - 1.2j, -0.8 - 0.1j, -0.4 + 0.8j,
                    0.8 + 1.0j, -0.5 + 0.1j, 0.1 + 1.2j, -1.7 - 0.9j)
        self.helper_with_lut(
            prec,
            src_data,
            digital.qam_16_0x0_0_1_2_3,
            digital.sd_qam_16_0x0_0_1_2_3)

    def test_constellation_soft_decoder_cf_qam16_8(self):
        prec = 8
        src_data = (-1.0 - 1.0j, 1.0 - 1.0j, -1.0 + 1.0j, 1.0 + 1.0j,
                    -2.0 - 2.0j, 2.0 - 2.0j, -2.0 + 2.0j, 2.0 + 2.0j,
                    -0.2 - 0.2j, 0.2 - 0.2j, -0.2 + 0.2j, 0.2 + 0.2j,
                    0.3 + 0.4j, 0.1 - 1.2j, -0.8 - 0.1j, -0.4 + 0.8j,
                    0.8 + 1.0j, -0.5 + 0.1j, 0.1 + 1.2j, -1.7 - 0.9j)
        self.helper_with_lut(
            prec,
            src_data,
            digital.qam_16_0x0_0_1_2_3,
            digital.sd_qam_16_0x0_0_1_2_3)

    def test_constellation_soft_decoder_cf_qam16_8_rand(self):
        prec = 8
        src_data = vectorize(complex)(
            2 * random.randn(100), 2 * random.randn(100))
        self.helper_with_lut(
            prec,
            src_data,
            digital.qam_16_0x0_0_1_2_3,
            digital.sd_qam_16_0x0_0_1_2_3,
            3)

    def test_constellation_soft_decoder_cf_qam16_8_rand2(self):
        prec = 8
        #src_data = vectorize(complex)(2*random.randn(100), 2*random.randn(100))
        src_data = vectorize(complex)(2 * random.randn(2), 2 * random.randn(2))
        self.helper_no_lut(
            prec,
            src_data,
            digital.qam_16_0x0_0_1_2_3,
            digital.sd_qam_16_0x0_0_1_2_3)


if __name__ == '__main__':
    gr_unittest.run(test_constellation_soft_decoder)



#!/usr/bin/env python
#
# Copyright 2021 Malte Lenhart
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#

from gnuradio import gr, gr_unittest
from gnuradio import blocks
import pmt
import time

# this test tests message strobe and message debug blocks against each other
# similar tests contained in message_debug class

# this tests the periodic message output and the input port to change the message


class qa_message_strobe(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_001_t(self):
        test_str = "test_msg"
        new_msg = "new_msg"
        message_period_ms = 100
        msg_strobe = blocks.message_strobe(
            pmt.intern(test_str), message_period_ms)
        msg_debug = blocks.message_debug()

        self.tb.msg_connect(msg_strobe, "strobe", msg_debug, "store")

        self.tb.start()

        self.assertAlmostEqual(msg_debug.num_messages(),
                               0, delta=2)  # 1st call, expect 0
        time.sleep(1.05)  # floor(1050/100) = 10
        self.assertAlmostEqual(msg_debug.num_messages(),
                               10, delta=8)  # 2nd call == 10
        time.sleep(1)  # floor(2050/100) = 20
        self.assertAlmostEqual(msg_debug.num_messages(),
                               20, delta=10)  # 3rd call == 20

        # change test message
        msg_strobe.to_basic_block()._post(pmt.intern("set_msg"), pmt.intern(new_msg))
        time.sleep(1)

        self.tb.stop()
        self.tb.wait()

        # check data
        # first received message matches initial test message
        self.assertAlmostEqual(pmt.to_python(msg_debug.get_message(
            0)), test_str, "mismatch initial test string")

        # last message matches changed test message
        no_msgs = msg_debug.num_messages()
        self.assertAlmostEqual(pmt.to_python(msg_debug.get_message(
            no_msgs - 1)), new_msg, "failed to update string")


if __name__ == '__main__':
    gr_unittest.run(qa_message_strobe)



#!/usr/bin/env python
#
# Copyright 2006,2007,2010,2011,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, digital, blocks
import pmt
import numpy
from time import sleep


class test_hdlc_framer(gr_unittest.TestCase):
    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_001(self):
        # test complementary operation of framer & deframer
        # want to frame some random data that has enough consecutive bits to
        # stuff at least a few bits
        npkts = 20
        src_data = [0xFE, 0xDA, 0xAC, 0x29, 0x7F, 0xA2, 0x90, 0x0F, 0xF8]
        frame = digital.hdlc_framer_pb("wat")
        deframe = digital.hdlc_deframer_bp(8, 500)
        debug = blocks.message_debug()
        self.tb.connect(frame, deframe)
        self.tb.msg_connect(deframe, "out", debug, "store")
        self.tb.start()
        msg = pmt.cons(pmt.PMT_NIL, pmt.init_u8vector(len(src_data), src_data))
        for i in range(npkts):
            frame.to_basic_block()._post(pmt.intern("in"), msg)
        sleep(0.2)
        self.tb.stop()
        self.tb.wait()
        rxmsg = debug.get_message(0)
        result_len = pmt.blob_length(pmt.cdr(rxmsg))
        msg_data = []
        for j in range(result_len):
            msg_data.append(pmt.u8vector_ref(pmt.cdr(rxmsg), j))
        self.assertEqual(src_data, msg_data)


if __name__ == '__main__':
    gr_unittest.run(test_hdlc_framer)
